@@ -32,6 +32,10 @@
  * 	  James
  * 26/6/15
  *	- better profile sanity checking for icc import
+ * 2/8/17
+ * 	- remove lcms1 support, it was untested
+ * 10/10/17
+ * 	- more input profile sanity tests
  */
 
 /*
@@ -66,7 +70,7 @@
 #endif /*HAVE_CONFIG_H*/
 #include <vips/intl.h>
 
-#if defined( HAVE_LCMS ) || defined( HAVE_LCMS2 )
+#ifdef HAVE_LCMS2
 
 #include <stdio.h>
 #include <math.h>
@@ -74,18 +78,7 @@
 
 /* Has to be before VIPS to avoid nameclashes.
  */
-#ifdef HAVE_LCMS2
 #include <lcms2.h>
-#else /*HAVE_LCMS*/
-#include <lcms.h>
-
-/* Use the lcms2 names.
- */
-#define cmsSigRgbData icSigRgbData 
-#define cmsSigLabData icSigLabData 
-#define cmsSigCmykData icSigCmykData 
-#define cmsSigXYZData icSigXYZData 
-#endif
 
 #include <vips/vips.h>
 
@@ -95,16 +88,6 @@
  */
 #define PIXEL_BUFFER_SIZE (10000)
 
-/* LCMS1 was missing some stuff.
- */
-#ifdef HAVE_LCMS
-typedef DWORD cmsUInt32Number;
-
-/* This doesn't exist in lcms1, just set it to zero.
- */
-#define cmsFLAGS_NOCACHE (0)
-#endif
-
 /**
  * VipsIntent:
  * @VIPS_INTENT_PERCEPTUAL: perceptual rendering intent
@@ -169,10 +152,6 @@ typedef struct _VipsIcc {
 	cmsUInt32Number out_icc_format;
 	cmsHTRANSFORM trans;
 
-	/* We need to single-thread calls to LCMS 1.
-	 */
-	GMutex *lock;
-
 } VipsIcc;
 
 typedef VipsColourCodeClass VipsIccClass;
@@ -182,24 +161,11 @@ G_DEFINE_ABSTRACT_TYPE( VipsIcc, vips_icc, VIPS_TYPE_COLOUR_CODE );
 /* Error from lcms.
  */
 
-#ifdef HAVE_LCMS2
 static void
 icc_error( cmsContext context, cmsUInt32Number code, const char *text )
 {
 	vips_error( "VipsIcc", "%s", text );
 }
-#else
-static int 
-icc_error( int code, const char *text )
-{
-	if( code == LCMS_ERRC_WARNING )
-		g_warning( "%s", text );
-	else
-		vips_error( "VipsIcc", text );
-
-	return( 0 );
-}
-#endif
 
 static void
 vips_icc_dispose( GObject *gobject )
@@ -209,7 +175,6 @@ vips_icc_dispose( GObject *gobject )
 	VIPS_FREEF( cmsDeleteTransform, icc->trans );
 	VIPS_FREEF( cmsCloseProfile, icc->in_profile );
 	VIPS_FREEF( cmsCloseProfile, icc->out_profile );
-	VIPS_FREEF( vips_g_mutex_free, icc->lock );
 
 	G_OBJECT_CLASS( vips_icc_parent_class )->dispose( gobject );
 }
@@ -251,7 +216,6 @@ vips_icc_build( VipsObject *object )
 				TYPE_RGB_16 : TYPE_RGB_8;
 			break;
 
-#ifdef HAVE_LCMS2
 		case cmsSigGrayData:
 			colour->input_bands = 1;
 			code->input_format = 
@@ -261,7 +225,6 @@ vips_icc_build( VipsObject *object )
 				code->in->BandFmt == VIPS_FORMAT_USHORT ? 
 				TYPE_GRAY_16 : TYPE_GRAY_8;
 			break;
-#endif /*HAVE_LCMS2*/
 
 		case cmsSigCmykData:
 			colour->input_bands = 4;
@@ -311,7 +274,6 @@ vips_icc_build( VipsObject *object )
 				TYPE_RGB_16 : TYPE_RGB_8;
 			break;
 
-#ifdef HAVE_LCMS2
 		case cmsSigGrayData:
 			colour->interpretation = 
 				icc->depth == 8 ? 
@@ -325,7 +287,6 @@ vips_icc_build( VipsObject *object )
 				icc->depth == 16 ? 
 				TYPE_GRAY_16 : TYPE_GRAY_8;
 			break;
-#endif /*HAVE_LCMS2*/
 
 		case cmsSigCmykData:
 			colour->interpretation = VIPS_INTERPRETATION_CMYK;
@@ -414,21 +375,12 @@ vips_icc_class_init( VipsIccClass *class )
 		G_STRUCT_OFFSET( VipsIcc, pcs ),
 		VIPS_TYPE_PCS, VIPS_PCS_LAB );
 
-#ifdef HAVE_LCMS2
 	cmsSetLogErrorHandler( icc_error );
-#else
-	/* Ask lcms not to abort on error.
-	 */
-	cmsErrorAction( LCMS_ERROR_IGNORE );
-	cmsSetErrorHandler( icc_error );
-#endif
-
 }
 
 static void
 vips_icc_init( VipsIcc *icc )
 {
-	icc->lock = vips_g_mutex_new();
 	icc->intent = VIPS_INTENT_RELATIVE;
 	icc->pcs = VIPS_PCS_LAB;
 	icc->depth = 8;
@@ -466,11 +418,9 @@ vips_icc_profile_needs_bands( cmsHPROFILE profile )
 	int needs_bands;
 
 	switch( cmsGetColorSpace( profile ) ) {
-#ifdef HAVE_LCMS2
 	case cmsSigGrayData:
 		needs_bands = 1;
 		break;
-#endif /*HAVE_LCMS2*/
 
 	case cmsSigRgbData:
 	case cmsSigLabData:
@@ -541,6 +491,63 @@ vips_image_expected_bands( VipsImage *image )
 	return( expected_bands );
 }
 
+/* What cmsColorSpaceSignature do we expect this image to be (roughly) after 
+ * preprocessing. Again, fragile :( see the FIXME above.
+ */
+static cmsColorSpaceSignature
+vips_image_expected_sig( VipsImage *image )
+{
+	cmsColorSpaceSignature expected_sig;
+
+	switch( image->Type ) { 
+	case VIPS_INTERPRETATION_B_W:
+	case VIPS_INTERPRETATION_GREY16:
+		expected_sig = cmsSigGrayData;
+		break;
+
+	case VIPS_INTERPRETATION_LAB:
+	case VIPS_INTERPRETATION_LABQ:
+	case VIPS_INTERPRETATION_LABS:
+		expected_sig = cmsSigLabData;
+		break;
+
+	case VIPS_INTERPRETATION_sRGB:
+	case VIPS_INTERPRETATION_RGB:
+	case VIPS_INTERPRETATION_RGB16:
+	case VIPS_INTERPRETATION_scRGB:
+		expected_sig = cmsSigRgbData;
+		break;
+
+	case VIPS_INTERPRETATION_XYZ:
+		expected_sig = cmsSigXYZData;
+		break;
+
+	case VIPS_INTERPRETATION_CMYK:
+		expected_sig = cmsSigCmykData;
+		break;
+
+	case VIPS_INTERPRETATION_HSV:
+		expected_sig = cmsSigHsvData;
+		break;
+
+	case VIPS_INTERPRETATION_YXY:
+		expected_sig = cmsSigYxyData;
+		break;
+
+	case VIPS_INTERPRETATION_LCH:
+	case VIPS_INTERPRETATION_CMC:
+	case VIPS_INTERPRETATION_MULTIBAND:
+	case VIPS_INTERPRETATION_HISTOGRAM:
+	case VIPS_INTERPRETATION_MATRIX:
+	case VIPS_INTERPRETATION_FOURIER:
+	default:
+		expected_sig = -1;
+		break;
+	}
+
+	return( expected_sig );
+}
+
 static cmsHPROFILE
 vips_icc_load_profile_image( VipsImage *image )
 {
@@ -565,6 +572,12 @@ vips_icc_load_profile_image( VipsImage *image )
 			_( "embedded profile incompatible with image" ) );
 		return( NULL );
 	}
+	if( vips_image_expected_sig( image ) != cmsGetColorSpace( profile ) ) {
+		VIPS_FREEF( cmsCloseProfile, profile );
+		g_warning( "%s", 
+			_( "embedded profile colourspace differs from image" ) );
+		return( NULL );
+	}
 
 	return( profile );
 }
@@ -588,6 +601,12 @@ vips_icc_load_profile_file( const char *domain,
 			filename );
 		return( NULL );
 	}
+	if( vips_image_expected_sig( image ) != cmsGetColorSpace( profile ) ) {
+		VIPS_FREEF( cmsCloseProfile, profile );
+		g_warning( _( "profile \"%s\" colourspace "
+			"differs from image" ), filename );
+		return( NULL );
+	}
 
 	return( profile );
 }
@@ -631,14 +650,10 @@ vips_icc_import_build( VipsObject *object )
 		icc->in_profile, icc->intent, LCMS_USED_AS_INPUT );
 
 	if( icc->pcs == VIPS_PCS_LAB ) { 
-#ifdef HAVE_LCMS2
 		cmsCIExyY white;
 		cmsWhitePointFromTemp( &white, 6500 );
 
 		icc->out_profile = cmsCreateLab4Profile( &white );
-#else
-		icc->out_profile = cmsCreateLabProfile( NULL );
-#endif
 	}
 	else 
 		icc->out_profile = cmsCreateXYZProfile();
@@ -704,13 +719,7 @@ vips_icc_import_line( VipsColour *colour,
 	for( i = 0; i < width; i += PIXEL_BUFFER_SIZE ) {
 		const int chunk = VIPS_MIN( width - i, PIXEL_BUFFER_SIZE );
 
-#ifdef HAVE_LCMS2
 		cmsDoTransform( icc->trans, p, encoded, chunk );
-#else
-		g_mutex_lock( icc->lock );
-		cmsDoTransform( icc->trans, p, encoded, chunk );
-		g_mutex_unlock( icc->lock );
-#endif
 
 		if( icc->pcs == VIPS_PCS_LAB ) 
 			decode_lab( encoded, q, chunk );
@@ -788,14 +797,10 @@ vips_icc_export_build( VipsObject *object )
 		icc->pcs = VIPS_PCS_XYZ; 
 
 	if( icc->pcs == VIPS_PCS_LAB ) { 
-#ifdef HAVE_LCMS2
 		cmsCIExyY white;
 		cmsWhitePointFromTemp( &white, 6500 );
 
 		icc->in_profile = cmsCreateLab4Profile( &white );
-#else
-		icc->in_profile = cmsCreateLabProfile( NULL );
-#endif
 	}
 	else 
 		icc->in_profile = cmsCreateXYZProfile();
@@ -940,13 +945,7 @@ vips_icc_export_line( VipsColour *colour,
 		else
 			encode_xyz( p, encoded, chunk );
 
-#ifdef HAVE_LCMS2
 		cmsDoTransform( icc->trans, encoded, q, chunk );
-#else
-		g_mutex_lock( icc->lock );
-		cmsDoTransform( icc->trans, encoded, q, chunk );
-		g_mutex_unlock( icc->lock );
-#endif
 
 		p += PIXEL_BUFFER_SIZE * 3;
 		q += PIXEL_BUFFER_SIZE * VIPS_IMAGE_SIZEOF_PEL( colour->out );
@@ -1070,13 +1069,7 @@ vips_icc_transform_line( VipsColour *colour,
 {
 	VipsIcc *icc = (VipsIcc *) colour;
 
-#ifdef HAVE_LCMS2
 	cmsDoTransform( icc->trans, in[0], out, width );
-#else
-	g_mutex_lock( icc->lock );
-	cmsDoTransform( icc->trans, in[0], out, width );
-	g_mutex_unlock( icc->lock );
-#endif
 }
 
 static void
@@ -1132,9 +1125,9 @@ vips_icc_transform_init( VipsIccTransform *transform )
 }
 
 /**
- * vips_icc_ac2rc:
+ * vips_icc_ac2rc: (method)
  * @in: input image
- * @out: output image
+ * @out: (out): output image
  * @profile_filename: use this profile
  *
  * Transform an image from absolute to relative colorimetry using the
@@ -1149,6 +1142,7 @@ vips_icc_ac2rc( VipsImage *in, VipsImage **out, const char *profile_filename )
 {
 	VipsImage *t;
 	cmsHPROFILE profile;
+	cmsCIEXYZ *media;
 	double X, Y, Z;
 	double *add;
 	double *mul;
@@ -1157,10 +1151,6 @@ vips_icc_ac2rc( VipsImage *in, VipsImage **out, const char *profile_filename )
 	if( !(profile = cmsOpenProfileFromFile( profile_filename, "r" )) )
 		return( -1 );
 
-#ifdef HAVE_LCMS2
-{
-	cmsCIEXYZ *media;
-
 	if( !(media = cmsReadTag( profile, cmsSigMediaWhitePointTag )) ) {
 		vips_error( "vips_icc_ac2rc", 
 			"%s", _( "unable to get media white point" ) );
@@ -1170,22 +1160,6 @@ vips_icc_ac2rc( VipsImage *in, VipsImage **out, const char *profile_filename )
 	X = media->X;
 	Y = media->Y;
 	Z = media->Z;
-}
-#else /*HAVE_LCMS*/
-{
-	cmsCIEXYZ media;
-
-	if( !cmsTakeMediaWhitePoint( &media, profile ) ) {
-		vips_error( "vips_icc_ac2rc", 
-			"%s", _( "unable to get media white point" ) );
-		return( -1 );
-	}
-
-	X = media.X;
-	Y = media.Y;
-	Z = media.Z;
-}
-#endif
 
 	cmsCloseProfile( profile );
 
@@ -1223,7 +1197,7 @@ vips_icc_ac2rc( VipsImage *in, VipsImage **out, const char *profile_filename )
 	return( 0 );
 }
 
-#else /*!HAVE_LCMS*/
+#else /*!HAVE_LCMS2*/
 
 #include <vips/vips.h>
 
@@ -1245,9 +1219,9 @@ vips_icc_ac2rc( VipsImage *in, VipsImage **out, const char *profile_filename )
 #endif /*HAVE_LCMS*/
 
 /**
- * vips_icc_import:
+ * vips_icc_import: (method)
  * @in: input image
- * @out: output image
+ * @out: (out): output image
  * @...: %NULL-terminated list of optional named arguments
  *
  * Optional arguments:
@@ -1288,9 +1262,9 @@ vips_icc_import( VipsImage *in, VipsImage **out, ... )
 }
 
 /**
- * vips_icc_export:
+ * vips_icc_export: (method)
  * @in: input image
- * @out: output image
+ * @out: (out): output image
  * @...: %NULL-terminated list of optional named arguments
  *
  * Optional arguments:
@@ -1323,9 +1297,9 @@ vips_icc_export( VipsImage *in, VipsImage **out, ... )
 }
 
 /**
- * vips_icc_transform:
+ * vips_icc_transform: (method)
  * @in: input image
- * @out: output image
+ * @out: (out): output image
  * @output_profile: get the output profile from here
  * @...: %NULL-terminated list of optional named arguments
  *
