@@ -14,12 +14,16 @@
  * 	- rewrite as a class
  * 20/9/15 leiyangyou 
  * 	- add @spacing 
+ * 29/5/17
+ * 	- don't set "font" if unset, it breaks caching
+ * 16/7/17 gargsms
+ * 	- implement auto fitting of text inside bounds
  */
 
 /*
 
     This file is part of VIPS.
-    
+
     VIPS is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -43,6 +47,10 @@
 
  */
 
+/*
+#define DEBUG
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif /*HAVE_CONFIG_H*/
@@ -66,6 +74,7 @@ typedef struct _VipsText {
 	char *text;
 	char *font;
 	int width;
+	int height;
 	int spacing;
 	VipsAlign align;
 	int dpi;
@@ -107,7 +116,7 @@ vips_text_dispose( GObject *gobject )
 static PangoLayout *
 text_layout_new( PangoContext *context, 
 	const char *text, const char *font, int width, int spacing,
-	VipsAlign align, int dpi )
+	VipsAlign align )
 {
 	PangoLayout *layout;
 	PangoFontDescription *font_description;
@@ -148,6 +157,157 @@ text_layout_new( PangoContext *context,
 	return( layout );
 }
 
+static int
+vips_text_get_extents( VipsText *text, VipsRect *extents )
+{
+	PangoRectangle logical_rect;
+
+	pango_ft2_font_map_set_resolution( 
+		PANGO_FT2_FONT_MAP( vips_text_fontmap ), text->dpi, text->dpi );
+	text->context = pango_font_map_create_context( 
+		PANGO_FONT_MAP( vips_text_fontmap ) );
+
+	if( !(text->layout = text_layout_new( text->context, 
+		text->text, text->font, 
+		text->width, text->spacing, text->align )) ) 
+		return( -1 );
+
+	pango_layout_get_extents( text->layout, NULL, &logical_rect );
+	extents->left = PANGO_PIXELS( logical_rect.x );
+	extents->top = PANGO_PIXELS( logical_rect.y );
+	extents->width = PANGO_PIXELS( logical_rect.width );
+	extents->height = PANGO_PIXELS( logical_rect.height );
+
+#ifdef DEBUG
+	printf( "vips_text_get_extents: dpi = %d, "
+		"left = %d, top = %d, width = %d, height = %d\n",
+		text->dpi, 
+		extents->left, extents->top, extents->width, extents->height );
+#endif /*DEBUG*/
+
+	return( 0 );
+}
+
+/* Return -ve for extents too small, 0 for a fit, +ve for extents too large.
+ */
+static int
+vips_text_rect_difference( VipsRect *target, VipsRect *extents )
+{
+	if( vips_rect_includesrect( target, extents ) ) {
+		if( target->width == extents->width ||
+			target->height == extents->height )
+			return( 0 );
+
+		return( -1 );
+	}
+	else
+		return( 1 );
+}
+
+/* Adjust text->dpi to try to fit to the bounding box.
+ */
+static int
+vips_text_autofit( VipsText *text, VipsRect *out_extents ) 
+{
+	VipsRect target;
+	VipsRect extents;
+	VipsRect previous_extents;
+	int difference;
+	int previous_difference;
+	int previous_dpi;
+
+	int lower_dpi;
+	int upper_dpi;
+	VipsRect lower_extents;
+	VipsRect upper_extents;
+
+	/* First, repeatedly double or halve dpi until we pass the correct
+	 * value. This will give us a lower and upper bound.
+	 */
+	target.width = text->width;
+	target.height = text->height;
+	previous_dpi = -1;
+
+	for(;;) { 
+		if( vips_text_get_extents( text, &extents ) )
+			return( -1 ); 
+		target.left = extents.left;
+		target.top = extents.top;
+		difference = vips_text_rect_difference( &target, &extents );
+
+		if( previous_dpi == -1 ) {
+			previous_dpi = text->dpi;
+			previous_difference = difference;
+			previous_extents = extents;
+		}
+
+		/* Hit the size, or we straddle the target.
+		 */
+		if( difference == 0 ||
+			difference != previous_difference ) 
+			break;
+
+		previous_difference = difference;
+		previous_extents = extents;
+		previous_dpi = text->dpi;
+
+		text->dpi = difference < 0 ? text->dpi * 2 : text->dpi / 2;
+	}
+
+	if( difference < 0 ) {
+		/* We've been coming down.
+		 */
+		lower_dpi = text->dpi;
+		lower_extents = extents;
+		upper_dpi = previous_dpi;
+		upper_extents = previous_extents;
+	}
+	else {
+		lower_dpi = previous_dpi;
+		lower_extents = previous_extents;
+		upper_dpi = text->dpi;
+		upper_extents = extents;
+	}
+
+	/* Refine lower and upper until they are almost touching.
+	 */
+	while( upper_dpi - lower_dpi > 1 &&
+		difference != 0 ) { 
+		text->dpi = (upper_dpi + lower_dpi) / 2;
+		if( vips_text_get_extents( text, &extents ) )
+			return( -1 ); 
+		target.left = extents.left;
+		target.top = extents.top;
+		difference = vips_text_rect_difference( &target, &extents );
+
+		if( difference < 0 ) { 
+			lower_dpi = text->dpi;
+			lower_extents = extents;
+		}
+		else {
+			upper_dpi = text->dpi;
+			upper_extents = extents;
+		}
+	}
+
+	/* If we've hit the target exactly and quit the loop, diff will be 0
+	 * and we can use upper. Otherwise we are straddling the target and we
+	 * must take lower.
+	 */
+	if( difference == 0 ) {
+		text->dpi = upper_dpi;
+		*out_extents = upper_extents;
+	}
+	else {
+		text->dpi = lower_dpi;
+		*out_extents = lower_extents;
+	}
+
+	g_object_set( text, "autofit_dpi", text->dpi, NULL ); 
+
+	return( 0 ); 
+}
+
 static int
 vips_text_build( VipsObject *object )
 {
@@ -155,11 +315,7 @@ vips_text_build( VipsObject *object )
 	VipsCreate *create = VIPS_CREATE( object );
 	VipsText *text = (VipsText *) object;
 
-	PangoRectangle logical_rect;
-	int left;
-	int top;
-	int width;
-	int height;
+	VipsRect extents;
 	int y;
 
 	if( VIPS_OBJECT_CLASS( vips_text_parent_class )->build( object ) )
@@ -171,52 +327,35 @@ vips_text_build( VipsObject *object )
 		return( -1 );
 	}
 
-	if( !text->font )
-		g_object_set( text, "font", "sans 12", NULL ); 
-
 	g_mutex_lock( vips_text_lock ); 
 
 	if( !vips_text_fontmap )
 		vips_text_fontmap = pango_ft2_font_map_new();
 
-	pango_ft2_font_map_set_resolution( 
-		PANGO_FT2_FONT_MAP( vips_text_fontmap ), text->dpi, text->dpi );
-	text->context = pango_font_map_create_context( 
-		PANGO_FONT_MAP( vips_text_fontmap ) );
-
-	if( !(text->layout = text_layout_new( text->context, 
-		text->text, text->font, 
-		text->width, text->spacing, text->align, text->dpi )) ) {
-		g_mutex_unlock( vips_text_lock ); 
-		return( -1 );
+	/* If our caller set height and not dpi, we adjust dpi until 
+	 * we get a fit.
+	 */
+	if( vips_object_argument_isset( object, "height" ) &&
+		!vips_object_argument_isset( object, "dpi" ) ) {
+		if( vips_text_autofit( text, &extents ) )
+			return( -1 );
 	}
-
-	pango_layout_get_extents( text->layout, NULL, &logical_rect );
-
-#ifdef DEBUG
-	printf( "logical left = %d, top = %d, width = %d, height = %d\n",
-		PANGO_PIXELS( logical_rect.x ),
-		PANGO_PIXELS( logical_rect.y ),
-		PANGO_PIXELS( logical_rect.width ),
-		PANGO_PIXELS( logical_rect.height ) );
-#endif /*DEBUG*/
-
-	left = PANGO_PIXELS( logical_rect.x );
-	top = PANGO_PIXELS( logical_rect.y );
-	width = PANGO_PIXELS( logical_rect.width );
-	height = PANGO_PIXELS( logical_rect.height );
+	else
+		if( vips_text_get_extents( text, &extents ) )
+			return( -1 );
 
 	/* Can happen for "", for example.
 	 */
-	if( width == 0 || height == 0 ) {
+	if( extents.width == 0 || 
+		extents.height == 0 ) {
 		vips_error( class->nickname, "%s", _( "no text to render" ) );
 		g_mutex_unlock( vips_text_lock ); 
 		return( -1 );
 	}
 
-	text->bitmap.width = width;
+	text->bitmap.width = extents.width;
 	text->bitmap.pitch = (text->bitmap.width + 3) & ~3;
-	text->bitmap.rows = height;
+	text->bitmap.rows = extents.height;
 	if( !(text->bitmap.buffer = 
 		VIPS_ARRAY( NULL, 
 			text->bitmap.pitch * text->bitmap.rows, VipsPel )) ) {
@@ -230,7 +369,7 @@ vips_text_build( VipsObject *object )
 
 	if( pango_layout_get_width( text->layout ) != -1 )
 		pango_ft2_render_layout( &text->bitmap, text->layout, 
-			-left, -top );
+			-extents.left, -extents.top );
 	else
 		pango_ft2_render_layout( &text->bitmap, text->layout, 0, 0 );
 
@@ -269,7 +408,7 @@ vips_text_class_init( VipsTextClass *class )
 
 	static GOnce once = G_ONCE_INIT;
 
-	(void) g_once( &once, vips_text_make_lock, NULL );
+	VIPS_ONCE( &once, vips_text_make_lock, NULL );
 
 	gobject_class->dispose = vips_text_dispose;
 	gobject_class->set_property = vips_object_set_property;
@@ -300,21 +439,35 @@ vips_text_class_init( VipsTextClass *class )
 		G_STRUCT_OFFSET( VipsText, width ),
 		0, VIPS_MAX_COORD, 0 );
 
-	VIPS_ARG_ENUM( class, "align", 7, 
+	VIPS_ARG_INT( class, "height", 7, 
+		_( "Height" ), 
+		_( "Maximum image height in pixels" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsText, height ),
+		0, VIPS_MAX_COORD, 0 );
+
+	VIPS_ARG_ENUM( class, "align", 8, 
 		_( "Align" ), 
 		_( "Align on the low, centre or high edge" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsText, align ),
-		VIPS_TYPE_ALIGN, VIPS_ALIGN_LOW ); 
+		VIPS_TYPE_ALIGN, VIPS_ALIGN_LOW );
 
-	VIPS_ARG_INT( class, "dpi", 8, 
+	VIPS_ARG_INT( class, "dpi", 9, 
 		_( "DPI" ), 
 		_( "DPI to render at" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsText, dpi ),
 		1, 1000000, 72 );
 
-	VIPS_ARG_INT( class, "spacing", 9, 
+	VIPS_ARG_INT( class, "autofit_dpi", 10, 
+		_( "Autofit DPI" ), 
+		_( "DPI selected by autofit" ),
+		VIPS_ARGUMENT_OPTIONAL_OUTPUT,
+		G_STRUCT_OFFSET( VipsText, dpi ),
+		1, 1000000, 72 );
+
+	VIPS_ARG_INT( class, "spacing", 11, 
 		_( "Spacing" ), 
 		_( "Line spacing" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
@@ -329,13 +482,14 @@ vips_text_init( VipsText *text )
 	text->align = VIPS_ALIGN_LOW;
 	text->dpi = 72;
 	text->bitmap.buffer = NULL;
+	VIPS_SETSTR( text->font, "sans 12" ); 
 }
 
 #endif /*HAVE_PANGOFT2*/
 
 /**
  * vips_text:
- * @out: output image
+ * @out: (out): output image
  * @text: utf-8 text string to render
  * @...: %NULL-terminated list of optional named arguments
  *
@@ -343,8 +497,10 @@ vips_text_init( VipsText *text )
  *
  * * @font: %gchararray, font to render with
  * * @width: %gint, image should be no wider than this many pixels
+ * * @height: %gint, image should be no higher than this many pixels
  * * @align: #VipsAlign, left/centre/right alignment
  * * @dpi: %gint, render at this resolution
+ * * @autofit_dpi: %gint, read out auto-fitted DPI 
  * * @spacing: %gint, space lines by this in points
  *
  * Draw the string @text to an image. @out is a one-band 8-bit
@@ -357,13 +513,21 @@ vips_text_init( VipsText *text )
  * @font is the font to render with, as a fontconfig name. Examples might be
  * "sans 12" or perhaps "bitstream charter bold 10".
  *
- * @width is the maximum number of pixels across to draw within. If the
- * generated text is wider than this, it will wrap to a new line. In this
- * case, @align can be used to set the alignment style for multi-line
- * text. 
+ * @width is the number of pixels to word-wrap at. Lines of text wider than
+ * this will be broken at word bounaries. 
+ * @align can be used to set the alignment style for multi-line
+ * text. Note that the output image can be wider than @width if there are no
+ * word breaks. 
+ *
+ * @height is the maximum number of pixels high the generated text can be. This
+ * only takes effect when @dpi is not set, and @width is set, making a box. 
+ * In this case, vips_text() will search for a @dpi which will just fit the 
+ * text into @width and @height.
+ *
+ * You can use @autofit_dpi to read out the DPI selected by auto fit. 
  *
  * @dpi sets the resolution to render at. "sans 12" at 72 dpi draws characters
- * approximately 12 pixels high.
+ * approximately 12 pixels high. 
  *
  * @spacing sets the line spacing, in points. It would typicallly be something
  * like font size times 1.2.
