@@ -16,6 +16,8 @@
  * 	- remove max-alpha stuff, this is now automatic
  * 12/6/17
  * 	- transform cmyk->rgb if there's an embedded profile
+ * 16/6/17
+ * 	- add page_height
  */
 
 /*
@@ -582,7 +584,7 @@ vips_foreign_find_load_buffer_sub( VipsForeignLoadClass *load_class,
  * vips_foreign_find_load_buffer:
  * @data: (array length=size) (element-type guint8) (transfer none): start of 
  * memory buffer
- * @size: number of bytes in @data
+ * @size: (type gsize): number of bytes in @data
  *
  * Searches for an operation you could use to load a memory buffer. To see the
  * range of buffer loaders supported by your vips, try something like:
@@ -640,8 +642,8 @@ vips_foreign_is_a( const char *loader, const char *filename )
 /**
  * vips_foreign_is_a_buffer:
  * @loader: name of loader to use for test
- * @data: pointer to the buffer to test
- * @size: size of the buffer to test
+ * @data: (array length=size) (element-type guint8): pointer to the buffer to test
+ * @size: (type gsize): size of the buffer to test
  *
  * Return %TRUE if @data can be loaded by @loader. @loader is something
  * like "tiffload_buffer" or "VipsForeignLoadTiffBuffer".
@@ -738,12 +740,18 @@ vips_foreign_load_temp( VipsForeignLoad *load )
 		return( vips_image_new() );
 	}
 
+	/* ->memory used to be called ->disc and default TRUE. If it's been
+	 * forced FALSE, set memory TRUE.
+	 */
+	if( !load->disc )
+		load->memory = TRUE;
+
 	/* We open via disc if:
-	 * - 'disc' is set
+	 * - 'memory' is off
 	 * - the uncompressed image will be larger than 
 	 *   vips_get_disc_threshold()
 	 */
-	if( load->disc && 
+	if( !load->memory && 
 		image_size > disc_threshold ) {
 #ifdef DEBUG
 		printf( "vips_foreign_load_temp: disc temp\n" );
@@ -995,12 +1003,12 @@ vips_foreign_load_class_init( VipsForeignLoadClass *class )
 		G_STRUCT_OFFSET( VipsForeignLoad, flags ),
 		VIPS_TYPE_FOREIGN_FLAGS, VIPS_FOREIGN_NONE ); 
 
-	VIPS_ARG_BOOL( class, "disc", 7, 
-		_( "Disc" ), 
-		_( "Open to disc" ),
+	VIPS_ARG_BOOL( class, "memory", 7, 
+		_( "Memory" ), 
+		_( "Force open via memory" ),
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
-		G_STRUCT_OFFSET( VipsForeignLoad, disc ),
-		TRUE );
+		G_STRUCT_OFFSET( VipsForeignLoad, memory ),
+		FALSE );
 
 	VIPS_ARG_ENUM( class, "access", 8, 
 		_( "Access" ), 
@@ -1023,6 +1031,13 @@ vips_foreign_load_class_init( VipsForeignLoadClass *class )
 		G_STRUCT_OFFSET( VipsForeignLoad, fail ),
 		FALSE );
 
+	VIPS_ARG_BOOL( class, "disc", 12, 
+		_( "Disc" ), 
+		_( "Open to disc" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT | VIPS_ARGUMENT_DEPRECATED,
+		G_STRUCT_OFFSET( VipsForeignLoad, disc ),
+		TRUE );
+
 }
 
 static void
@@ -1032,12 +1047,12 @@ vips_foreign_load_init( VipsForeignLoad *load )
 	load->access = VIPS_ACCESS_RANDOM;
 }
 
-/**
+/*
  * Loaders can call this
  */
 
 /**
- * vips_foreign_load_invalidate:
+ * vips_foreign_load_invalidate: (method)
  * @image: image to invalidate
  *
  * Loaders can call this on the image they are making if they see a read error
@@ -1485,6 +1500,10 @@ vips_foreign_save_build( VipsObject *object )
 			save->background ) )
 			return( -1 );
 
+		if( save->page_height )
+			vips_image_set_int( ready, 
+				VIPS_META_PAGE_HEIGHT, save->page_height );
+
 		VIPS_UNREF( save->ready );
 		save->ready = ready;
 	}
@@ -1570,6 +1589,13 @@ vips_foreign_save_class_init( VipsForeignSaveClass *class )
 		VIPS_ARGUMENT_OPTIONAL_INPUT,
 		G_STRUCT_OFFSET( VipsForeignSave, background ),
 		VIPS_TYPE_ARRAY_DOUBLE );
+
+	VIPS_ARG_INT( class, "page_height", 8, 
+		_( "Page height" ), 
+		_( "Set page height for multipage save" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsForeignSave, page_height ),
+		0, VIPS_MAX_COORD, 0 ); 
 }
 
 static void
