@@ -13,6 +13,11 @@
  * 	- better transparency detection, thanks diegocsandrim
  * 25/11/16
  * 	- support @n, page-height
+ * 5/10/17
+ * 	- colormap can be missing thanks Kleis
+ * 21/11/17
+ * 	- add "gif-delay", "gif-loop", "gif-comment" metadata
+ * 	- add dispose handling
  */
 
 /*
@@ -77,6 +82,15 @@
 #  endif
 #endif
 
+/* Added in giflib5.
+ */
+#ifndef HAVE_GIFLIB_5
+#define DISPOSAL_UNSPECIFIED      0       
+#define DISPOSE_DO_NOT            1      
+#define DISPOSE_BACKGROUND        2     
+#define DISPOSE_PREVIOUS          3    
+#endif
+
 typedef struct _VipsForeignLoadGif {
 	VipsForeignLoad parent_object;
 
@@ -115,6 +129,25 @@ typedef struct _VipsForeignLoadGif {
 	gboolean has_transparency;
 	gboolean has_colour;
 
+	/* Delay in 1/100ths of a second. We only track a single delay 
+	 * value for the whole file, and we report the first delay we see. Some
+	 * GIFs have a long delay on the final frame.
+	 */
+	gboolean has_delay;
+	int delay;
+
+	/* Number of times to loop the animation.
+	 */
+	int loop;
+
+	/* The GIF comment, if any.
+	 */
+	char *comment; 
+
+	/* The current dispose method.
+	 */
+	int dispose;
+
 	/* The FILE* we read from.
 	 */
 	FILE *fp;
@@ -310,6 +343,8 @@ vips_foreign_load_gif_dispose( GObject *gobject )
 
 	vips_foreign_load_gif_close( gif ); 
 
+	VIPS_FREE( gif->comment ); 
+
 	G_OBJECT_CLASS( vips_foreign_load_gif_parent_class )->
 		dispose( gobject );
 }
@@ -366,19 +401,31 @@ vips_foreign_load_gif_render_line( VipsForeignLoadGif *gif,
 
 	for( x = 0; x < width; x++ ) {
 		VipsPel v = p[x];
-
-		if( v >= map->ColorCount ) {
-			g_warning( "%s", _( "pixel value out of range" ) );
-			continue;
-		}
-
-		if( v != gif->transparency ) { 
+		
+		if( map &&
+			v < map->ColorCount &&
+			v != gif->transparency ) {
 			q[0] = map->Colors[v].Red;
 			q[1] = map->Colors[v].Green;
 			q[2] = map->Colors[v].Blue;
 			q[3] = 255;
 		}
+		else if( v != gif->transparency ) {
+			/* If there's no map, just save the index.
+			 */
+			q[0] = v;
+			q[1] = v;
+			q[2] = v;
+			q[3] = 255;
+		}
+		else if( gif->dispose == DISPOSE_DO_NOT ) {
+			/* Transparent pixels let the previous frame show
+			 * through, ie., do nothing.
+			 */
+		}
 		else {
+			/* All other modes are just transparent. 
+			 */
 			q[0] = 0;
 			q[1] = 0;
 			q[2] = 0;
@@ -389,12 +436,12 @@ vips_foreign_load_gif_render_line( VipsForeignLoadGif *gif,
 	}
 }
 
-/* Render the current gif frame into an RGBA buffer. GIFs 
- * accumulate, so don't clear the buffer first, we need to paint a 
- * series of frames on top of each other. 
+/* Render the current gif frame into an RGBA buffer. GIFs can accumulate, 
+ * depending on the current dispose mode.
  */
 static int
-vips_foreign_load_gif_render( VipsForeignLoadGif *gif, VipsImage *out ) 
+vips_foreign_load_gif_render( VipsForeignLoadGif *gif, 
+	VipsImage *previous, VipsImage *out )
 {
 	VipsObjectClass *class = VIPS_OBJECT_GET_CLASS( gif );
 	GifFileType *file = gif->file;
@@ -414,7 +461,8 @@ vips_foreign_load_gif_render( VipsForeignLoadGif *gif, VipsImage *out )
 
 	/* Check if we have a non-greyscale colourmap for this frame.
 	 */
-	if( !gif->has_colour ) {
+	if( !gif->has_colour &&
+		map ) {
 		int i;
 
 		for( i = 0; i < map->ColorCount; i++ ) 
@@ -487,11 +535,166 @@ vips_foreign_load_gif_render( VipsForeignLoadGif *gif, VipsImage *out )
 	return( 0 );
 }
 
+static int
+vips_foreign_load_gif_extension_next( VipsForeignLoadGif *gif,
+	GifByteType **extension )
+{
+	if( DGifGetExtensionNext( gif->file, extension ) == GIF_ERROR ) {
+		vips_foreign_load_gif_error( gif ); 
+		return( -1 ); 
+	}
+
+	if( *extension ) 
+		VIPS_DEBUG_MSG( "gifload: EXTENSION_NEXT\n" ); 
+
+	return( 0 );
+}
+
+static int
+vips_foreign_load_gif_graphics_ext( VipsForeignLoadGif *gif, 
+	GifByteType *extension )
+{
+	VIPS_DEBUG_MSG( "gifload: type: graphics\n" ); 
+
+	if( extension[0] == 4 ) { 
+		/* Bytes are flags, delay low, delay high,
+		 * transparency. Flag bit 1 means transparency
+		 * is being set.
+		 */
+		gif->transparency = -1;
+		if( extension[1] & 0x1 ) {
+			gif->transparency = extension[4];
+			gif->has_transparency = TRUE;
+			VIPS_DEBUG_MSG( "gifload: transparency %d\n", 
+				gif->transparency );
+		}
+
+		/* Set the current dispose mode. This is read during frame load
+		 * to set the meaning of background and transparent pixels.
+		 */
+		gif->dispose = (extension[1] >> 2) & 0x7;
+		VIPS_DEBUG_MSG( "gifload: dispose %d\n", gif->dispose );
+
+		if( !gif->has_delay ) { 
+			gif->has_delay = TRUE;
+			gif->delay = extension[2] | (extension[3] << 8);
+		}
+	}
+
+	while( extension != NULL ) 
+		if( vips_foreign_load_gif_extension_next( gif, &extension ) )
+			return( -1 ); 
+
+	return( 0 );
+}
+
+static int
+vips_foreign_load_gif_application_ext( VipsForeignLoadGif *gif,
+	GifByteType *extension )
+{
+	gboolean have_netscape;
+
+	VIPS_DEBUG_MSG( "gifload: type: application\n" ); 
+
+	/* The 11-byte NETSCAPE extension.
+	 */
+	have_netscape = FALSE;
+	if( extension[0] == 11 &&
+		vips_isprefix( "NETSCAPE2.0", (const char *) (extension + 1) ) ) 
+		have_netscape = TRUE;
+
+	while( extension != NULL ) {
+		if( vips_foreign_load_gif_extension_next( gif, &extension ) )
+			return( -1 ); 
+
+		if( have_netscape &&
+			extension &&
+			extension[0] == 3 &&
+			extension[1] == 1 ) {
+			gif->loop = extension[2] | (extension[3] << 8);
+			VIPS_DEBUG_MSG( "gifload: seen loop %d\n",
+				gif->loop ); 
+		}
+	}
+
+	return( 0 );
+}
+
+static int
+vips_foreign_load_gif_comment_ext( VipsForeignLoadGif *gif,
+	GifByteType *extension )
+{
+	VIPS_DEBUG_MSG( "gifload: type: comment\n" ); 
+
+	if( !gif->comment ) { 
+		/* Up to 257 with a NULL terminator.
+		 */
+		char comment[257];
+
+		vips_strncpy( comment, (char *) (extension + 1), 256 );
+		comment[extension[0]] = '\0';
+		gif->comment = g_strdup( comment ); 
+	}
+
+	while( extension != NULL ) 
+		if( vips_foreign_load_gif_extension_next( gif, &extension ) )
+			return( -1 ); 
+
+	return( 0 );
+}
+
+static int
+vips_foreign_load_gif_extension( VipsForeignLoadGif *gif )
+{
+	GifByteType *extension;
+	int ext_code;
+
+	VIPS_DEBUG_MSG( "gifload: EXTENSION_RECORD_TYPE\n" ); 
+
+	if( DGifGetExtension( gif->file, &ext_code, &extension ) == GIF_ERROR ) {
+		vips_foreign_load_gif_error( gif ); 
+		return( -1 ); 
+	}
+
+	if( extension ) 
+		switch( ext_code ) { 
+		case GRAPHICS_EXT_FUNC_CODE: 
+			if( vips_foreign_load_gif_graphics_ext( gif,
+				extension ) )
+				return( -1 ); 
+			break;
+
+		case APPLICATION_EXT_FUNC_CODE: 
+			if( vips_foreign_load_gif_application_ext( gif, 
+				extension ) )
+				return( -1 ); 
+			break;
+
+		case COMMENT_EXT_FUNC_CODE:
+			if( vips_foreign_load_gif_comment_ext( gif, extension ) )
+				return( -1 ); 
+			break;
+
+		default:
+			/* Step over any NEXT blocks for unknown extensions.
+			 */
+			while( extension != NULL ) 
+				if( vips_foreign_load_gif_extension_next( gif, 
+					&extension ) )
+					return( -1 ); 
+			break;
+		}
+
+	return( 0 );
+}
+
 /* Write the next page, if there is one, to @page. Set EOF if we hit the end of
- * the file. @page must be a memory image of the right size. 
+ * the file. @page must be a memory image of the right size. @previous is the
+ * previous frame, if any. 
  */
 static int
-vips_foreign_load_gif_page( VipsForeignLoadGif *gif, VipsImage *out )
+vips_foreign_load_gif_page( VipsForeignLoadGif *gif, 
+	VipsImage *previous, VipsImage *out )
 {
 	GifRecordType record;
 	int n_pages;
@@ -499,9 +702,6 @@ vips_foreign_load_gif_page( VipsForeignLoadGif *gif, VipsImage *out )
 	n_pages = 0;
 
 	do { 
-		GifByteType *extension;
-		int ext_code;
-
 		if( DGifGetRecordType( gif->file, &record ) == GIF_ERROR ) {
 			vips_foreign_load_gif_error( gif ); 
 			return( -1 ); 
@@ -509,77 +709,39 @@ vips_foreign_load_gif_page( VipsForeignLoadGif *gif, VipsImage *out )
 
 		switch( record ) {
 		case IMAGE_DESC_RECORD_TYPE:
-			VIPS_DEBUG_MSG( "gifload: IMAGE_DESC_RECORD_TYPE:\n" ); 
+			VIPS_DEBUG_MSG( "gifload: IMAGE_DESC_RECORD_TYPE\n" ); 
 
 			if( DGifGetImageDesc( gif->file ) == GIF_ERROR ) {
 				vips_foreign_load_gif_error( gif ); 
 				return( -1 ); 
 			}
 
-			if( vips_foreign_load_gif_render( gif, out ) )
+			if( vips_foreign_load_gif_render( gif, previous, out ) )
 				return( -1 ); 
 
 			n_pages += 1;
 
-			VIPS_DEBUG_MSG( "gifload: page %d:\n", 
+			VIPS_DEBUG_MSG( "gifload: page %d\n", 
 				gif->current_page + n_pages );
 
 			break;
 
 		case EXTENSION_RECORD_TYPE:
-			VIPS_DEBUG_MSG( "gifload: EXTENSION_RECORD_TYPE:\n" ); 
-
-			gif->transparency = -1;
-
-			if( DGifGetExtension( gif->file, 
-				&ext_code, &extension ) == GIF_ERROR ) {
-				vips_foreign_load_gif_error( gif ); 
-				return( -1 ); 
-			}
-
-			if( ext_code == GRAPHICS_EXT_FUNC_CODE &&
-				extension &&
-				extension[0] == 4 && 
-				(extension[1] & 0x1) ) {
-				/* Bytes are 4, 1, delay low, delay high,
-				 * transparency. Bit 1 means transparency
-				 * is being set.
-				 */
-				gif->transparency = extension[4];
-				gif->has_transparency = TRUE;
-
-				VIPS_DEBUG_MSG( "gifload: "
-					"seen transparency %d\n", 
-					gif->transparency );
-			}
-
-			while( extension != NULL ) {
-				if( DGifGetExtensionNext( gif->file, 
-					&extension) == GIF_ERROR ) {
-					vips_foreign_load_gif_error( gif ); 
-					return( -1 ); 
-				}
-
-#ifdef VIPS_DEBUG
-				if( extension ) 
-					VIPS_DEBUG_MSG( "gifload: "
-						"EXTENSION_NEXT:\n" ); 
-#endif
-			}
-
+			if( vips_foreign_load_gif_extension( gif ) )
+				return( -1 );
 			break;
 
 		case TERMINATE_RECORD_TYPE:
-			VIPS_DEBUG_MSG( "gifload: TERMINATE_RECORD_TYPE:\n" ); 
+			VIPS_DEBUG_MSG( "gifload: TERMINATE_RECORD_TYPE\n" ); 
 			gif->eof = TRUE;
 			break;
 
 		case SCREEN_DESC_RECORD_TYPE:
-			VIPS_DEBUG_MSG( "gifload: SCREEN_DESC_RECORD_TYPE:\n" );
+			VIPS_DEBUG_MSG( "gifload: SCREEN_DESC_RECORD_TYPE\n" );
 			break;
 
 		case UNDEFINED_RECORD_TYPE:
-			VIPS_DEBUG_MSG( "gifload: UNDEFINED_RECORD_TYPE:\n" );
+			VIPS_DEBUG_MSG( "gifload: UNDEFINED_RECORD_TYPE\n" );
 			break;
 
 		default:
@@ -662,7 +824,7 @@ vips_foreign_load_gif_pages( VipsForeignLoadGif *gif, VipsImage **out )
 	if( !(frame = vips_foreign_load_gif_new_page( gif )) ) 
 		return( -1 );
 	do { 
-		if( vips_foreign_load_gif_page( gif, frame ) ) {
+		if( vips_foreign_load_gif_page( gif, NULL, frame ) ) {
 			g_object_unref( frame );
 			return( -1 );
 		}
@@ -688,14 +850,19 @@ vips_foreign_load_gif_pages( VipsForeignLoadGif *gif, VipsImage **out )
 			return( -1 );
 		}
 
-		/* And init with the previous frame, if any.
-		 */
-		if( previous ) 
+		if( gif->dispose == DISPOSE_BACKGROUND )
+			/* BACKGROUND means the bg shows through, ie. (in web
+			 * terms) everything is transparent.
+			 */
+			memset( VIPS_IMAGE_ADDR( frame, 0, 0 ),
+				0,
+				VIPS_IMAGE_SIZEOF_IMAGE( frame ) );
+		else 
 			memcpy( VIPS_IMAGE_ADDR( frame, 0, 0 ),
 				VIPS_IMAGE_ADDR( previous, 0, 0 ),
 				VIPS_IMAGE_SIZEOF_IMAGE( frame ) );
 
-		if( vips_foreign_load_gif_page( gif, frame ) ) {
+		if( vips_foreign_load_gif_page( gif, previous, frame ) ) {
 			g_object_unref( frame ); 
 			unref_array( frames );
 			return( -1 );
@@ -709,7 +876,12 @@ vips_foreign_load_gif_pages( VipsForeignLoadGif *gif, VipsImage **out )
 		}
 		else {
 			frames = g_slist_append( frames, frame );
-			previous = frame;
+
+			/* These two dispose modes set new background frames.
+			 */
+			if( gif->dispose == DISPOSAL_UNSPECIFIED ||
+				gif->dispose == DISPOSE_DO_NOT ) 
+				previous = frame;
 		}
 	}
 
@@ -747,7 +919,11 @@ vips_foreign_load_gif_pages( VipsForeignLoadGif *gif, VipsImage **out )
 	unref_array( frames );
 
 	if( n_frames > 1 )
-		vips_image_set_int( *out, VIPS_META_PAGE_HEIGHT, frame->Ysize );
+		vips_image_set_int( *out, VIPS_META_PAGE_HEIGHT, t[0]->Ysize );
+	vips_image_set_int( *out, "gif-delay", gif->delay );
+	vips_image_set_int( *out, "gif-loop", gif->loop );
+	if( gif->comment ) 
+		vips_image_set_string( *out, "gif-comment", gif->comment );
 
 	return( 0 );
 }
@@ -845,6 +1021,10 @@ vips_foreign_load_gif_init( VipsForeignLoadGif *gif )
 {
 	gif->n = 1;
 	gif->transparency = -1;
+	gif->delay = 4;
+	gif->loop = 0;
+	gif->comment = NULL;
+	gif->dispose = 0;
 }
 
 typedef struct _VipsForeignLoadGifFile {
@@ -1004,7 +1184,7 @@ vips_foreign_load_gif_buffer_init( VipsForeignLoadGifBuffer *buffer )
 /**
  * vips_gifload:
  * @filename: file to load
- * @out: output image
+ * @out: (out): output image
  * @...: %NULL-terminated list of optional named arguments
  *
  * Optional arguments:
@@ -1043,9 +1223,9 @@ vips_gifload( const char *filename, VipsImage **out, ... )
 
 /**
  * vips_gifload_buffer:
- * @buf: memory area to load
- * @len: size of memory area
- * @out: image to write
+ * @buf: (array length=len) (element-type guint8): memory area to load
+ * @len: (type gsize): size of memory area
+ * @out: (out): image to write
  * @...: %NULL-terminated list of optional named arguments
  *
  * Optional arguments:
