@@ -171,6 +171,11 @@
  * 29/1/17
  * 	- enable bigtiff automatically for large, uncompressed writes, thanks 
  * 	  AndreasSchmid1 
+ * 26/8/17
+ * 	- support pyramid creation to buffer, thanks bubba
+ * 24/10/17
+ * 	- no error on page-height not a factor of image height, just don't
+ * 	  write multipage
  */
 
 /*
@@ -325,7 +330,8 @@ embed_profile_file( TIFF *tif, const char *profile )
 	char *buffer;
 	size_t length;
 
-	if( !(buffer = vips__file_read_name( profile, VIPS_ICC_DIR, &length )) )
+	if( !(buffer = vips__file_read_name( profile, 
+		vips__icc_dir(), &length )) )
 		return( -1 );
 	TIFFSetField( tif, TIFFTAG_ICCPROFILE, length, buffer );
 	vips_free( buffer );
@@ -929,33 +935,23 @@ wtiff_new( VipsImage *im, const char *filename,
 		return( NULL );
 	}
 
-	/* If page_height <= Ysize, treat as a single-page image.
+	/* If page_height <= Ysize and it's a factor of image height, save 
+	 * as a toilet roll image.
 	 */
 	if( wtiff->page_height > 0 &&
-		wtiff->page_height < im->Ysize ) { 
+		wtiff->page_height < im->Ysize &&
+		im->Ysize % wtiff->page_height == 0 ) { 
 #ifdef DEBUG
 		printf( "wtiff_new: detected toilet roll image, "
 			"page-height=%d\n", 
 			wtiff->page_height );
+		printf( "wtiff_new: pages=%d\n", 
+			im->Ysize / wtiff->page_height );
 #endif/*DEBUG*/
 
 		wtiff->toilet_roll = TRUE;
 		wtiff->image_height = wtiff->page_height;
 
-		if( im->Ysize % wtiff->page_height != 0 ) { 
-			vips_error( "vips2tiff",
-				_( "image height %d is not a factor of "
-					"page-height %d" ),
-				im->Ysize, wtiff->page_height );
-			wtiff_free( wtiff );
-			return( NULL );
-		}
-
-#ifdef DEBUG
-		printf( "wtiff_new: pages=%d\n", 
-			im->Ysize / wtiff->page_height );
-#endif/*DEBUG*/
-
 		/* We can't pyramid toilet roll images.
 		 */
 		if( wtiff->pyramid ) {
@@ -1664,8 +1660,16 @@ wtiff_gather( Wtiff *wtiff )
 			printf( "Appending layer %s ...\n", layer->lname );
 #endif /*DEBUG*/
 
-			if( !(in = vips__tiff_openin( layer->lname )) ) 
-				return( -1 );
+			if( layer->lname ) {
+				if( !(in = vips__tiff_openin( layer->lname )) ) 
+					return( -1 );
+			}
+			else {
+				if( !(in = vips__tiff_openin_buffer( wtiff->im,
+					layer->buf, layer->len )) ) 
+					return( -1 );
+			}
+
 			if( wtiff_copy_tiff( wtiff, wtiff->layer->tif, in ) ) {
 				TIFFClose( in );
 				return( -1 );
