@@ -478,12 +478,17 @@ png2vips_interlace( Read *read, VipsImage *out )
 
 	if( setjmp( png_jmpbuf( read->pPng ) ) ) 
 		return( -1 );
-
+ 
 	if( !(read->row_pointer = VIPS_ARRAY( NULL, out->Ysize, png_bytep )) )
 		return( -1 );
 	for( y = 0; y < out->Ysize; y++ )
 		read->row_pointer[y] = VIPS_IMAGE_ADDR( out, 0, y );
 
+	/* Some libpng warn you to call png_set_interlace_handling(); here, but
+	 * that can actually break interlace. We have to live with the warning,
+	 * unfortunately.
+	 */
+
 	png_read_image( read->pPng, read->row_pointer );
 
 	png_read_end( read->pPng, NULL ); 
@@ -954,7 +959,7 @@ write_vips( Write *write,
 			size_t length;
 
 			if( !(data = vips__file_read_name( profile, 
-				VIPS_ICC_DIR, &length )) ) 
+				vips__icc_dir(), &length )) ) 
 				return( -1 );
 
 #ifdef DEBUG
