@@ -12,6 +12,8 @@
  * 	- vips_image_write() does not ref input for non-partial images
  * 29/10/16
  * 	- add vips_image_hasalpha()
+ * 11/10/1
+ * 	- more severing for vips_image_write()
  */
 
 /*
@@ -119,7 +121,7 @@
  * to formatted memory buffers (with vips_image_write_to_buffer()) and to
  * C-style memory arrays (with vips_image_write_to_memory().
  *
- * You can also write image to other images. Create, for example, a temporary
+ * You can also write images to other images. Create, for example, a temporary
  * disc image with vips_image_new_temp_file(), then write your image to that
  * with vips_image_write(). You can create several other types of image and
  * write to them, see vips_image_new_memory(), for example. 
@@ -771,18 +773,18 @@ vips_image_preeval_cb( VipsImage *image, VipsProgress *progress, int *last )
 {
 	int tile_width; 
 	int tile_height; 
-	int nlines;
+	int n_lines;
 
 	*last = -1;
 
 	vips_get_tile_size( image, 
-		&tile_width, &tile_height, &nlines );
+		&tile_width, &tile_height, &n_lines );
 	printf( _( "%s %s: %d x %d pixels, %d threads, %d x %d tiles, "
 		"%d lines in buffer" ),
 		g_get_prgname(), image->filename,
 		image->Xsize, image->Ysize,
 		vips_concurrency_get(),
-		tile_width, tile_height, nlines );
+		tile_width, tile_height, n_lines );
 	printf( "\n" );
 }
 
@@ -1413,7 +1415,7 @@ vips_image_invalidate_all_cb( VipsImage *image )
 }
 
 /**
- * vips_image_invalidate_all:
+ * vips_image_invalidate_all: (method)
  * @image: #VipsImage to invalidate
  *
  * Invalidate all pixel caches on @image and any downstream images, that
@@ -1453,7 +1455,7 @@ vips_image_minimise_all_cb( VipsImage *image )
 }
 
 /**
- * vips_image_minimise_all:
+ * vips_image_minimise_all: (method)
  * @image: #VipsImage to minimise
  *
  * Minimise memory use on this image and any upstream images, that is, images
@@ -1591,7 +1593,7 @@ vips_image_posteval( VipsImage *image )
 }
 
 /**
- * vips_image_set_progress:
+ * vips_image_set_progress: (method)
  * @image: image to signal progress on
  * @progress: turn progress reporting on or off
  *
@@ -1615,7 +1617,7 @@ vips_image_set_progress( VipsImage *image, gboolean progress )
 
 
 /**
- * vips_image_iskilled:
+ * vips_image_iskilled: (method)
  * @image: image to test
  *
  * If @image has been killed (see vips_image_set_kill()), set an error message,
@@ -1652,7 +1654,7 @@ vips_image_iskilled( VipsImage *image )
 }
 
 /**
- * vips_image_set_kill:
+ * vips_image_set_kill: (method)
  * @image: image to test
  * @kill: the kill state
  *
@@ -1688,7 +1690,7 @@ vips_image_temp_name( void )
 }
 
 /**
- * vips_image_new:
+ * vips_image_new: (constructor)
  *
  * vips_image_new() creates a new, empty #VipsImage. 
  * If you write to one of these images, vips will just attach some callbacks,
@@ -1744,7 +1746,7 @@ vips_image_new_mode( const char *filename, const char *mode )
 }
 
 /**
- * vips_image_new_memory:
+ * vips_image_new_memory: (constructor)
  *
  * vips_image_new_memory() creates a new #VipsImage which, when written to, will
  * create a memory image. 
@@ -1760,7 +1762,7 @@ vips_image_new_memory( void )
 }
 
 /**
- * vips_image_memory:
+ * vips_image_memory: (constructor)
  *
  * A renamed vips_image_new_memory() ... Some gobject binding systems do not 
  * like more than one _new() method.
@@ -1824,14 +1826,14 @@ vips_filename_get_options( const char *vips_filename )
 }
 
 /**
- * vips_image_new_from_file:
+ * vips_image_new_from_file: (constructor)
  * @name: file to open
  * @...: %NULL-terminated list of optional named arguments
  *
  * Optional arguments:
  *
  * * @access: hint #VipsAccess mode to loader
- * * @disc: load via a temporary disc file
+ * * @memory: force load via memory 
  *
  * vips_image_new_from_file() opens @name for reading. It can load files
  * in many image formats, including VIPS, TIFF, PNG, JPEG, FITS, Matlab,
@@ -1856,12 +1858,15 @@ vips_filename_get_options( const char *vips_filename )
  *
  * In #VIPS_ACCESS_RANDOM mode, small images are decompressed to memory and
  * then processed from there. Large images are decompressed to temporary
- * random-access files on disc and then processed from there. Set @disc to
- * %TRUE to force loading via disc. See vips_image_new_temp_file() for an 
+ * random-access files on disc and then processed from there. 
+ *
+ * Set @memory to %TRUE to force loading via memory. The default is to load 
+ * large random access images via temporary disc files. See 
+ * vips_image_new_temp_file() for an 
  * explanation of how VIPS selects a location for the temporary file.
  *
  * The disc threshold can be set with the "--vips-disc-threshold"
- * command-line argument, or the VIPS_DISC_THRESHOLD environment variable.
+ * command-line argument, or the `VIPS_DISC_THRESHOLD` environment variable.
  * The value is a simple integer, but can take a unit postfix of "k", 
  * "m" or "g" to indicate kilobytes, megabytes or gigabytes.
  * The default threshold is 100 MB.
@@ -1921,7 +1926,7 @@ vips_image_new_from_file( const char *name, ... )
 }
 
 /**
- * vips_image_new_from_file_RW:
+ * vips_image_new_from_file_RW: (constructor)
  * @filename: filename to open
  *
  * Opens the named file for simultaneous reading and writing. This will only 
@@ -1939,7 +1944,7 @@ vips_image_new_from_file_RW( const char *filename )
 }
 
 /**
- * vips_image_new_from_file_raw:
+ * vips_image_new_from_file_raw: (constructor)
  * @filename: filename to open
  * @xsize: image width
  * @ysize: image height
@@ -1982,9 +1987,9 @@ vips_image_new_from_file_raw( const char *filename,
 }
 
 /**
- * vips_image_new_from_memory:
+ * vips_image_new_from_memory: (constructor)
  * @data: (array length=size) (element-type guint8) (transfer none): start of memory area
- * @size: length of memory area
+ * @size: (type gsize): length of memory area
  * @width: image width
  * @height: image height
  * @bands: image bands (or bytes per pixel)
@@ -2057,9 +2062,9 @@ vips_image_new_from_memory_copy_cb( VipsImage *image, void *data_copy )
 }
 
 /**
- * vips_image_new_from_memory_copy:
+ * vips_image_new_from_memory_copy: (constructor)
  * @data: (array length=size) (element-type guint8) (transfer none): start of memory area
- * @size: length of memory area
+ * @size: (type gsize): length of memory area
  * @width: image width
  * @height: image height
  * @bands: image bands (or bytes per pixel)
@@ -2099,9 +2104,9 @@ vips_image_new_from_memory_copy( const void *data, size_t size,
 }
 
 /**
- * vips_image_new_from_buffer:
+ * vips_image_new_from_buffer: (constructor)
  * @buf: (array length=len) (element-type guint8) (transfer none): image data
- * @len: length of memory buffer
+ * @len: (type gsize): length of memory buffer
  * @option_string: set of extra options as a string
  * @...: %NULL-terminated list of optional named arguments
  *
@@ -2155,7 +2160,7 @@ vips_image_new_from_buffer( const void *buf, size_t len,
 }
 
 /**
- * vips_image_new_matrix:
+ * vips_image_new_matrix: (constructor)
  * @width: image width
  * @height: image height
  *
@@ -2201,7 +2206,7 @@ vips_image_new_matrix( int width, int height )
 }
 
 /**
- * vips_image_new_matrixv:
+ * vips_image_new_matrixv: (constructor)
  * @width: image width
  * @height: image height
  * @...: matrix coefficients
@@ -2235,11 +2240,11 @@ vips_image_new_matrixv( int width, int height, ... )
 }
 
 /**
- * vips_image_new_matrix_from_array:
+ * vips_image_new_matrix_from_array: (constructor)
  * @width: image width
  * @height: image height
  * @array: (array length=size) (transfer none): array of elements
- * @size: number of elements
+ * @size: (type gsize): number of elements
  *
  * A binding-friendly version of vips_image_new_matrixv().
  *
@@ -2273,11 +2278,11 @@ vips_image_new_matrix_from_array( int width, int height,
 }
 
 /**
- * vips_image_matrix_from_array:
+ * vips_image_matrix_from_array: (constructor)
  * @width: image width
  * @height: image height
  * @array: (array length=size) (transfer none): array of elements
- * @size: number of elements
+ * @size: (type gsize): number of elements
  *
  * A renamed vips_image_new_matrix_from_array(). Some gobject bindings do not
  * like more than one _new method.
@@ -2293,7 +2298,81 @@ vips_image_matrix_from_array( int width, int height,
 }
 
 /**
- * vips_image_set_delete_on_close:
+ * vips_image_new_from_image: (constructor)
+ * @image: image to copy
+ * @c: (array length=n) (transfer none): array of constants
+ * @n: number of constants
+ *
+ * Creates a new image with width, height, format, interpretation, resolution
+ * and offset taken from @image, but with number of bands taken from @n and the
+ * value of each band element set from @c.
+ *
+ * See also: vips_image_new_from_image1()
+ *
+ * Returns: (transfer full): the new #VipsImage, or %NULL on error.
+ */
+VipsImage *
+vips_image_new_from_image( VipsImage *image, const double *c, int n )
+{
+	VipsObject *scope = (VipsObject *) vips_image_new();
+	VipsImage **t = (VipsImage **) vips_object_local_array( scope, 5 );
+
+	double *ones;
+	int i;
+	VipsImage *result;
+
+	if( !(ones = VIPS_ARRAY( scope, n, double )) ) {
+		g_object_unref( scope );
+		return( NULL );
+	}
+	for( i = 0; i < n; i++ )
+		ones[i] = 1.0;
+
+	if( vips_black( &t[0], 1, 1, NULL ) ||
+		vips_linear( t[0], &t[1], ones, (double *) c, n, NULL ) ||
+		vips_cast( t[1], &t[2], image->BandFmt, NULL ) ||
+		vips_embed( t[2], &t[3], 0, 0, image->Xsize, image->Ysize,
+			"extend", VIPS_EXTEND_COPY, NULL ) ||
+		vips_copy( t[3], &t[4], 
+			"interpretation", image->Type,
+			"xres", image->Xres,
+			"yres", image->Yres,
+			"xoffset", image->Xoffset,
+			"yoffset", image->Yoffset,
+			NULL ) ) {
+		g_object_unref( scope );
+		return( NULL );
+	}
+
+	result = t[4];
+	g_object_ref( result );
+
+	g_object_unref( scope );
+
+	return( result ); 
+}
+
+/**
+ * vips_image_new_from_image1: (constructor)
+ * @image: image to copy
+ * @c: constants
+ *
+ * Creates a new image with width, height, format, interpretation, resolution
+ * and offset taken from @image, but with one band and each pixel having the
+ * value @c.
+ *
+ * See also: vips_image_new_from_image()
+ *
+ * Returns: (transfer full): the new #VipsImage, or %NULL on error.
+ */
+VipsImage *
+vips_image_new_from_image1( VipsImage *image, double c )
+{
+	return( vips_image_new_from_image( image, (const double *) &c, 1 ) );
+}
+
+/**
+ * vips_image_set_delete_on_close: (method)
  * @image: image to set
  * @delete_on_close: format of file
  *
@@ -2358,7 +2437,7 @@ vips_get_disc_threshold( void )
 }
 
 /**
- * vips_image_new_temp_file:
+ * vips_image_new_temp_file: (constructor)
  * @format: format of file
  *
  * Make a #VipsImage which, when written to, will create a temporary file on
@@ -2424,9 +2503,9 @@ vips_image_write_gen( VipsRegion *or,
 }
 
 /**
- * vips_image_write:
+ * vips_image_write: (method)
  * @image: image to write
- * @out: write to this image
+ * @out: (out): write to this image
  *
  * Write @image to @out. Use vips_image_new() and friends to create the
  * #VipsImage you want to write to.
@@ -2451,21 +2530,24 @@ vips_image_write( VipsImage *image, VipsImage *out )
 	/* If @out is a partial image, we need to make sure that @image stays
 	 * alive as long as @out is alive.
 	 *
-	 * If it's not partial, perhaps a file we write to, or a memory image,
-	 * it's fine for @image to go away.
+	 * If it's not partial, perhaps a file we write to or a memory image,
+	 * we need to break any links between @image and @out created by
+	 * vips_image_pipelinev().
 	 */
 	if( vips_image_ispartial( out ) ) { 
 		g_object_ref( image );
 		vips_object_local( out, image );
 	}
 	else {
+		vips__reorder_clear( image );
+		vips__link_break_all( out );
 	}
 
 	return( 0 );
 }
 
 /**
- * vips_image_write_to_file:
+ * vips_image_write_to_file: (method)
  * @image: image to write
  * @name: write to this file
  * @...: %NULL-terminated list of optional named arguments
@@ -2503,11 +2585,11 @@ vips_image_write_to_file( VipsImage *image, const char *name, ... )
 }
 
 /**
- * vips_image_write_to_buffer:
+ * vips_image_write_to_buffer: (method)
  * @in: image to write
  * @suffix: format to write 
  * @buf: (array length=size) (element-type guint8) (transfer full): return buffer start here
- * @size: return buffer length here
+ * @size: (type gsize): return buffer length here
  * @...: %NULL-terminated list of optional named arguments
  *
  * Writes @in to a memory buffer in a format specified by @suffix. 
@@ -2561,7 +2643,7 @@ vips_image_write_to_buffer( VipsImage *in,
 }
 
 /**
- * vips_image_write_to_memory:
+ * vips_image_write_to_memory: (method)
  * @in: image to write
  * @size: return buffer length here
  *
@@ -2606,9 +2688,9 @@ vips_image_write_to_memory( VipsImage *in, size_t *size_out )
 }
 
 /**
- * vips_image_decode:
+ * vips_image_decode: (method)
  * @in: image to decode
- * @out: write to this image
+ * @out: (out): write to this image
  *
  * A convenience function to unpack to a format that we can compute with. 
  * @out.coding is always #VIPS_CODING_NONE. 
@@ -2642,10 +2724,10 @@ vips_image_decode( VipsImage *in, VipsImage **out )
 }
 
 /**
- * vips_image_decode_predict:
+ * vips_image_decode_predict: (method)
  * @in: image to decode
- * @bands: predict bands here
- * @format: predict format here
+ * @bands: (out): predict bands here
+ * @format: (out): predict format here
  *
  * We often need to know what an image will decode to without actually
  * decoding it, for example, in arg checking.
@@ -2681,9 +2763,9 @@ vips_image_decode_predict( VipsImage *in,
 }
 
 /**
- * vips_image_encode:
+ * vips_image_encode: (method)
  * @in: image to encode
- * @out: write to this image
+ * @out: (out): write to this image
  * @coding: coding to apply
  *
  * A convenience function to pack to a coding. The inverse of
@@ -2713,7 +2795,7 @@ vips_image_encode( VipsImage *in, VipsImage **out, VipsCoding coding )
 }
 
 /**
- * vips_image_isMSBfirst:
+ * vips_image_isMSBfirst: (method)
  * @image: image to test
  *
  * Return %TRUE if @image is in most-significant-
@@ -2730,7 +2812,7 @@ vips_image_isMSBfirst( VipsImage *image )
 }
 
 /**
- * vips_image_isfile:
+ * vips_image_isfile: (method)
  * @image: image to test
  *
  * Return %TRUE if @image represents a file on disc in some way. 
@@ -2757,7 +2839,7 @@ gboolean vips_image_isfile( VipsImage *image )
 }
 
 /**
- * vips_image_ispartial:
+ * vips_image_ispartial: (method)
  * @image: image to test
  *
  * Return %TRUE if @im represents a partial image (a delayed calculation).
@@ -2772,7 +2854,7 @@ vips_image_ispartial( VipsImage *image )
 }
 
 /**
- * vips_image_hasalpha:
+ * vips_image_hasalpha: (method)
  * @image: image to check
  *
  * libvips assumes an image has an alpha if it has two bands (ie. it is a
@@ -2790,7 +2872,7 @@ vips_image_hasalpha( VipsImage *image )
 }
 
 /**
- * vips_image_write_prepare:
+ * vips_image_write_prepare: (method)
  * @image: image to prepare
  *
  * Call this after setting header fields (width, height, and so on) to
@@ -2855,7 +2937,7 @@ vips_image_write_prepare( VipsImage *image )
 }
 
 /**
- * vips_image_write_line:
+ * vips_image_write_line: (method)
  * @image: image to write to
  * @ypos: vertical position of scan-line to write
  * @linebuffer: scanline of pixels
@@ -2991,15 +3073,15 @@ vips_image_rewind_output( VipsImage *image )
 }
 
 /** 
- * vips_image_copy_memory:
+ * vips_image_copy_memory: (method)
  * @image: image to copy to a memory buffer
  *
- * Allocate a memory buffer and copy @image to it. This is a thread-safe
- * equivalent of vips_image_wio_input(), useful if @image is small and from an
- * unknown source.
+ * Make an image which is an area of memory. 
  *
- * If @image is already in memory (perhaps a mmaped file on disc),
- * vips_image_copy_memory() will just ref @image and return that.
+ * If @image is already a memory buffer, just ref and return. If it's a file on
+ * disc or a partial, allocate memory and copy the image to it. 
+ *
+ * This operation is thread-safe, unlike vips_image_wio_input(). 
  *
  * If you are sure that @image is not shared with another thread (perhaps you
  * have made it yourself), use vips_image_wio_input() instead.
@@ -3012,9 +3094,6 @@ VipsImage *
 vips_image_copy_memory( VipsImage *image )
 {
 	VipsImage *new;
-	VipsImage *image_array[2];
-	size_t length;
-	void *data;
 
 	switch( image->dtype ) {
 	case VIPS_IMAGE_SETBUF:
@@ -3030,36 +3109,11 @@ vips_image_copy_memory( VipsImage *image )
 	case VIPS_IMAGE_OPENOUT:
 	case VIPS_IMAGE_OPENIN:
 	case VIPS_IMAGE_PARTIAL:
-		/* We don't use vips_image_new_memory() and vips_image_write()
-		 * since we want to make a break in the pipeline and we want
-		 * to avoid all the machinery around reordering and dependency
-		 * links.
-		 *
-		 * We especially want to be able to unref input and have output
-		 * survive. Things like the progress signal must be cleared,
-		 * for example. 
-		 */
-
-		/* Write to a new memory image.
-		 */
-		if( !(data = vips_image_write_to_memory( image, &length )) )
-			return( NULL );
-		if( !(new = vips_image_new_from_memory( data, length,
-			image->Xsize, image->Ysize, image->Bands, 
-			image->BandFmt )) ) {
-			g_free( data );
-			return( NULL );
+		new = vips_image_new_memory();
+		if( vips_image_write( image, new ) ) {
+			g_object_unref( new );
+			return( NULL ); 
 		}
-
-		/* Copy over other fields and the metadata.
-		 */
-		image_array[0] = image;
-		image_array[1] = NULL;
-		if( vips__image_copy_fields_array( new, image_array ) ) {
-			VIPS_UNREF( new );
-			return( NULL );
-		}
-
 		break;
 
 	default:
@@ -3072,7 +3126,7 @@ vips_image_copy_memory( VipsImage *image )
 }
 
 /**
- * vips_image_wio_input:
+ * vips_image_wio_input: (method)
  * @image: image to transform
  *
  * Check that an image is readable via the VIPS_IMAGE_ADDR() macro, that is,
@@ -3247,7 +3301,7 @@ vips__image_wio_output( VipsImage *image )
 }
  
 /**
- * vips_image_inplace:
+ * vips_image_inplace: (method)
  * @image: image to make read-write
  *
  * Gets @image ready for an in-place operation, such as vips_draw_circle().
@@ -3304,7 +3358,7 @@ vips_image_inplace( VipsImage *image )
 }
 
 /**
- * vips_image_pio_input:
+ * vips_image_pio_input: (method)
  * @image: image to check
  *
  * Check that an image is readable with vips_region_prepare() and friends. 
@@ -3381,7 +3435,7 @@ vips_image_pio_input( VipsImage *image )
 }
 
 /**
- * vips_image_pio_output:
+ * vips_image_pio_output: (method)
  * @image: image to check
  *
  * Check that an image is writeable with vips_image_generate(). If it isn't,
