@@ -770,7 +770,7 @@ vips_object_get_argument( VipsObject *object, const char *name,
 }
 
 /**
- * vips_object_argument_isset:
+ * vips_object_argument_isset: 
  * @object: the object to fetch the args from
  * @name: arg to fetch
  *
@@ -793,7 +793,7 @@ vips_object_argument_isset( VipsObject *object, const char *name )
 }
 
 /**
- * vips_object_get_argument_flags:
+ * vips_object_get_argument_flags: 
  * @object: the object to fetch the args from
  * @name: arg to fetch
  *
@@ -816,7 +816,7 @@ vips_object_get_argument_flags( VipsObject *object, const char *name )
 }
 
 /**
- * vips_object_get_argument_priority:
+ * vips_object_get_argument_priority: 
  * @object: the object to fetch the args from
  * @name: arg to fetch
  *
@@ -1915,7 +1915,7 @@ vips_object_set_argument_from_string( VipsObject *object,
 		/* Setting gvalue will have upped @array_image's count again,
 		 * go back to 1 so that gvalue has the only ref.
 		 */
-		vips_area_unref( (VipsArea *) array_image );
+		vips_area_unref( VIPS_AREA( array_image ) );
 	}
 	else if( g_type_is_a( otype, VIPS_TYPE_OBJECT ) &&
 		(oclass = g_type_class_ref( otype )) ) { 
@@ -2268,7 +2268,7 @@ vips_object_set_valist( VipsObject *object, va_list ap )
  * |[
  * vips_object_set (operation,
  *   "input", in,
- *   "output", &amp;out,
+ *   "output", &out,
  *   NULL);
  * ]|
  *
@@ -2814,7 +2814,7 @@ vips_type_find( const char *basename, const char *nickname )
 	GType base;
 	GType type;
 
-	g_once( &once, (GThreadFunc) vips_class_build_hash, NULL ); 
+	VIPS_ONCE( &once, (GThreadFunc) vips_class_build_hash, NULL ); 
 
 	hit = (NicknameGType *) 
 		g_hash_table_lookup( vips__object_nickname_table, 
@@ -2908,9 +2908,9 @@ vips_object_local_array_cb( GObject *parent, VipsObjectLocal *local )
  *
  * t = vips_object_local_array( a, 5 );
  * if( 
- *   vips_add( a, b, &amp;t[0], NULL ) ||
- *   vips_invert( t[0], &amp;t[1], NULL ) ||
- *   vips_add( t[1], t[0], &amp;t[2], NULL ) ||
+ *   vips_add( a, b, &t[0], NULL ) ||
+ *   vips_invert( t[0], &t[1], NULL ) ||
+ *   vips_add( t[1], t[0], &t[2], NULL ) ||
  *   vips_costra( t[2], out, NULL ) )
  *   return( -1 );
  * ]|
