@@ -508,7 +508,7 @@ vips_region_init( VipsRegion *region )
 }
 
 /**
- * vips_region_new:
+ * vips_region_new: (constructor)
  * @image: image to create this region on
  *
  * Create a region. #VipsRegion s start out empty, you need to call 
@@ -553,7 +553,7 @@ vips_region_new( VipsImage *image )
  */
 
 /**
- * vips_region_buffer:
+ * vips_region_buffer: (method)
  * @reg: region to operate upon
  * @r: #VipsRect of pixels you need to be able to address
  *
@@ -563,7 +563,7 @@ vips_region_new( VipsImage *image )
  * Returns: 0 on success, or -1 for error.
  */
 int
-vips_region_buffer( VipsRegion *reg, VipsRect *r )
+vips_region_buffer( VipsRegion *reg, const VipsRect *r )
 {
 	VipsImage *im = reg->im;
 
@@ -622,18 +622,18 @@ vips_region_buffer( VipsRegion *reg, VipsRect *r )
 }
 
 /**
- * vips_region_image:
+ * vips_region_image: (method)
  * @reg: region to operate upon
  * @r: #VipsRect of pixels you need to be able to address
  *
- * The region is transformed so that at least @r pixels are available to be read from
- * image. The image needs to be a memory buffer or represent a file
- * on disc that has been mapped or can be mapped. 
+ * The region is transformed so that at least @r pixels are available to be 
+ * read from the image. The image needs to be a memory buffer or represent a 
+ * file on disc that has been mapped or can be mapped. 
  *
  * Returns: 0 on success, or -1 for error.
  */
 int
-vips_region_image( VipsRegion *reg, VipsRect *r )
+vips_region_image( VipsRegion *reg, const VipsRect *r )
 {
 	VipsImage *image = reg->im;
 
@@ -709,7 +709,7 @@ vips_region_image( VipsRegion *reg, VipsRect *r )
 }
 
 /**
- * vips_region_region:
+ * vips_region_region: (method)
  * @reg: region to operate upon
  * @dest: region to connect to
  * @r: #VipsRect of pixels you need to be able to address
@@ -725,16 +725,15 @@ vips_region_image( VipsRegion *reg, VipsRect *r )
  * Performs all clipping necessary to ensure that @reg->valid is indeed
  * valid.
  *
- * If the region we attach to is moved or destroyed, we can be left with dangling 
- * pointers! If the region we attach to is on another image, the two images 
- * must have 
- * the same sizeof( pel ).
+ * If the region we attach to is moved or destroyed, we can be left with 
+ * dangling pointers! If the region we attach to is on another image, the 
+ * two images must have the same sizeof( pel ).
  *
  * Returns: 0 on success, or -1 for error.
  */
 int
 vips_region_region( VipsRegion *reg, 
-	VipsRegion *dest, VipsRect *r, int x, int y )
+	VipsRegion *dest, const VipsRect *r, int x, int y )
 {
 	VipsRect image;
 	VipsRect wanted;
@@ -829,7 +828,7 @@ vips_region_region( VipsRegion *reg,
  * Do two regions point to the same piece of image? ie. 
  *
  * |[
- * 	VIPS_REGION_ADDR( reg1, x, y ) == VIPS_REGION_ADDR( reg2, x, y ) &amp;&amp;
+ * 	VIPS_REGION_ADDR( reg1, x, y ) == VIPS_REGION_ADDR( reg2, x, y ) &&
  * 	*VIPS_REGION_ADDR( reg1, x, y ) == 
  * 		*VIPS_REGION_ADDR( reg2, x, y ) for all x, y, reg1, reg2.
  * ]|
@@ -845,7 +844,7 @@ vips_region_equalsregion( VipsRegion *reg1, VipsRegion *reg2 )
 }
 
 /**
- * vips_region_position:
+ * vips_region_position: (method)
  * @reg: region to operate upon
  * @x: position to move to
  * @y: position to move to
@@ -885,7 +884,8 @@ vips_region_position( VipsRegion *reg, int x, int y )
 }
 
 int
-vips_region_fill( VipsRegion *reg, VipsRect *r, VipsRegionFillFn fn, void *a )
+vips_region_fill( VipsRegion *reg, 
+	const VipsRect *r, VipsRegionFillFn fn, void *a )
 {
 	g_assert( reg->im->dtype == VIPS_IMAGE_PARTIAL );
 	g_assert( reg->im->generate_fn );
@@ -928,7 +928,7 @@ vips_region_fill( VipsRegion *reg, VipsRect *r, VipsRegionFillFn fn, void *a )
 }
 
 /**
- * vips_region_paint:
+ * vips_region_paint: (method)
  * @reg: region to operate upon
  * @r: area to paint
  * @value: value to paint
@@ -947,7 +947,7 @@ vips_region_fill( VipsRegion *reg, VipsRect *r, VipsRegionFillFn fn, void *a )
  * See also: vips_region_black().
  */
 void
-vips_region_paint( VipsRegion *reg, VipsRect *r, int value )
+vips_region_paint( VipsRegion *reg, const VipsRect *r, int value )
 {
 	VipsRect clipped;
 
@@ -997,7 +997,7 @@ vips_region_paint( VipsRegion *reg, VipsRect *r, int value )
 }
 
 /**
- * vips_region_paint_pel:
+ * vips_region_paint_pel: (method)
  * @reg: region to operate upon
  * @r: area to paint
  * @ink: value to paint
@@ -1011,7 +1011,7 @@ vips_region_paint( VipsRegion *reg, VipsRect *r, int value )
  * See also: vips_region_paint().
  */
 void
-vips_region_paint_pel( VipsRegion *reg, VipsRect *r, VipsPel *ink )
+vips_region_paint_pel( VipsRegion *reg, const VipsRect *r, const VipsPel *ink )
 {
 	VipsRect ovl;
 
@@ -1048,7 +1048,7 @@ vips_region_paint_pel( VipsRegion *reg, VipsRect *r, VipsPel *ink )
 }
 
 /**
- * vips_region_black:
+ * vips_region_black: (method)
  * @reg: region to operate upon
  *
  * Paints 0 into the valid part of @reg.
@@ -1064,7 +1064,7 @@ vips_region_black( VipsRegion *reg )
 /**
  * vips_region_copy:
  * @reg: source region 
- * @dest: destination region 
+ * @dest: (inout): destination region 
  * @r: #VipsRect of pixels you need to copy
  * @x: postion of @r in @dest
  * @y: postion of @r in @dest
@@ -1076,7 +1076,8 @@ vips_region_black( VipsRegion *reg )
  * See also: vips_region_paint().
  */
 void
-vips_region_copy( VipsRegion *reg, VipsRegion *dest, VipsRect *r, int x, int y )
+vips_region_copy( VipsRegion *reg, 
+	VipsRegion *dest, const VipsRect *r, int x, int y )
 {
 	int z;
 	int len = VIPS_IMAGE_SIZEOF_PEL( reg->im ) * r->width;
@@ -1111,12 +1112,23 @@ vips_region_copy( VipsRegion *reg, VipsRegion *dest, VipsRect *r, int x, int y )
 		VIPS_IMAGE_SIZEOF_PEL( dest->im ) );
 #endif /*DEBUG*/
 
-	for( z = 0; z < r->height; z++ ) {
-		memcpy( q, p, len );
+	/* Copy the scanlines. 
+	 *
+	 * Special case: if the two sets of scanlines are end-to-end (this
+	 * happens if we are copying complete regions) we can do a single
+	 * memcpy() for the whole thing. This is a little faster since we 
+	 * won't have to do unaligned copies.
+	 */
+	if( len == plsk &&
+		len == qlsk ) 
+		memcpy( q, p, len * r->height );
+	else
+		for( z = 0; z < r->height; z++ ) {
+			memcpy( q, p, len );
 
-		p += plsk;
-		q += qlsk;
-	}
+			p += plsk;
+			q += qlsk;
+		}
 }
 
 /* Generate area @target in @to using pixels in @from. 
@@ -1124,7 +1136,8 @@ vips_region_copy( VipsRegion *reg, VipsRegion *dest, VipsRect *r, int x, int y )
  * VIPS_CODING_LABQ only.
  */
 static void
-vips_region_shrink_labpack( VipsRegion *from, VipsRegion *to, VipsRect *target )
+vips_region_shrink_labpack( VipsRegion *from, 
+	VipsRegion *to, const VipsRect *target )
 {
 	int ls = VIPS_REGION_LSKIP( from );
 
@@ -1149,7 +1162,7 @@ vips_region_shrink_labpack( VipsRegion *from, VipsRegion *to, VipsRect *target )
 			int b = sp[2] + sp[6] + 
 				sp[ls + 2] + sp[ls + 6];
 
-			q[0] = l >> 2;
+			q[0] = (l + 2) >> 2;
 			q[1] = a >> 2;
 			q[2] = b >> 2;
 			q[3] = 0;
@@ -1170,7 +1183,7 @@ vips_region_shrink_labpack( VipsRegion *from, VipsRegion *to, VipsRect *target )
 			int tot = tp[z] + tp[z + nb] +  \
 				tp1[z] + tp1[z + nb]; \
 			\
-			tq[z] = tot >> 2; \
+			tq[z] = (tot + 2) >> 2; \
 		} \
 		\
 		/* Move on two pels in input. \
@@ -1201,7 +1214,8 @@ vips_region_shrink_labpack( VipsRegion *from, VipsRegion *to, VipsRect *target )
 /* Generate area @target in @to using pixels in @from. Non-complex.
  */
 static void
-vips_region_shrink_uncoded( VipsRegion *from, VipsRegion *to, VipsRect *target )
+vips_region_shrink_uncoded( VipsRegion *from, 
+	VipsRegion *to, const VipsRect *target )
 {
 	int ls = VIPS_REGION_LSKIP( from );
 	int ps = VIPS_IMAGE_SIZEOF_PEL( from->im );
@@ -1284,7 +1298,8 @@ vips_region_shrink_uncoded( VipsRegion *from, VipsRegion *to, VipsRect *target )
  * last band as alpha.
  */
 static void
-vips_region_shrink_alpha( VipsRegion *from, VipsRegion *to, VipsRect *target )
+vips_region_shrink_alpha( VipsRegion *from, 
+	VipsRegion *to, const VipsRect *target )
 {
 	int ls = VIPS_REGION_LSKIP( from );
 	int nb = from->im->Bands;
@@ -1326,7 +1341,7 @@ vips_region_shrink_alpha( VipsRegion *from, VipsRegion *to, VipsRect *target )
 /**
  * vips_region_shrink:
  * @from: source region 
- * @to: destination region 
+ * @to: (inout): destination region 
  * @target: #VipsRect of pixels you need to copy
  *
  * Write the pixels @target in @to from the x2 larger area in @from.
@@ -1336,7 +1351,7 @@ vips_region_shrink_alpha( VipsRegion *from, VipsRegion *to, VipsRect *target )
  * See also: vips_region_copy().
  */
 int
-vips_region_shrink( VipsRegion *from, VipsRegion *to, VipsRect *target )
+vips_region_shrink( VipsRegion *from, VipsRegion *to, const VipsRect *target )
 {
 	VipsImage *image = from->im;
 
@@ -1387,7 +1402,7 @@ vips_region_generate( VipsRegion *reg )
 }
 
 /** 
- * vips_region_prepare:
+ * vips_region_prepare: (method)
  * @reg: region to prepare
  * @r: #VipsRect of pixels you need to be able to address
  *
@@ -1408,7 +1423,7 @@ vips_region_generate( VipsRegion *reg )
  * Returns: 0 on success, or -1 on error.
  */
 int
-vips_region_prepare( VipsRegion *reg, VipsRect *r )
+vips_region_prepare( VipsRegion *reg, const VipsRect *r )
 {	
 	VipsImage *im = reg->im;
 
@@ -1481,7 +1496,7 @@ vips_region_prepare( VipsRegion *reg, VipsRect *r )
  */
 static int
 vips_region_prepare_to_generate( VipsRegion *reg, 
-	VipsRegion *dest, VipsRect *r, int x, int y )
+	VipsRegion *dest, const VipsRect *r, int x, int y )
 {
 	VipsImage *im = reg->im;
 	VipsPel *p;
@@ -1515,9 +1530,9 @@ vips_region_prepare_to_generate( VipsRegion *reg,
 }
 
 /** 
- * vips_region_prepare_to:
+ * vips_region_prepare_to: (method)
  * @reg: region to prepare
- * @dest: region to write to
+ * @dest: (inout): region to write to
  * @r: #VipsRect of pixels you need to be able to address
  * @x: postion of @r in @dest
  * @y: postion of @r in @dest
@@ -1540,7 +1555,7 @@ vips_region_prepare_to_generate( VipsRegion *reg,
  */
 int
 vips_region_prepare_to( VipsRegion *reg, 
-	VipsRegion *dest, VipsRect *r, int x, int y )
+	VipsRegion *dest, const VipsRect *r, int x, int y )
 {
 	VipsImage *im = reg->im;
 	VipsRect image;
@@ -1675,7 +1690,7 @@ vips_region_prepare_to( VipsRegion *reg,
 /* Don't use this, use vips_reorder_prepare_many() instead.
  */
 int
-vips_region_prepare_many( VipsRegion **reg, VipsRect *r )
+vips_region_prepare_many( VipsRegion **reg, const VipsRect *r )
 {
 	for( ; *reg; ++reg )
 		if( vips_region_prepare( *reg, r ) )
@@ -1685,7 +1700,7 @@ vips_region_prepare_many( VipsRegion **reg, VipsRect *r )
 }
 
 /** 
- * vips_region_invalidate: 
+ * vips_region_invalidate: (method)
  * @reg: region to invalidate
  *
  * Mark a region as containing invalid pixels. Calling this function means
