@@ -543,6 +543,27 @@ vips__write( int fd, const void *buf, size_t count )
 	return( 0 );
 }
 
+#ifdef OS_WIN32
+/* Set the create date on a file. On Windows, the create date may be copied 
+ * over from an existing file of the same name, unless you reset it. 
+ *
+ * See https://blogs.msdn.microsoft.com/oldnewthing/20050715-14/?p=34923
+ */
+void
+vips__set_create_time( int fd )
+{
+	HANDLE handle;
+	SYSTEMTIME st;
+	FILETIME ft;
+
+	if( (handle = _get_osfhandle( fd )) == INVALID_HANDLE_VALUE )
+		return;
+	GetSystemTime( &st );
+	SystemTimeToFileTime( &st, &ft );
+	SetFileTime( handle, &ft, &ft, &ft );
+}
+#endif /*OS_WIN32*/
+
 /* open() with a utf8 filename, setting errno.
  */
 int
@@ -568,6 +589,9 @@ vips__open( const char *filename, int flags, ... )
 
 	fd = _wopen( path16, flags, mode );
 
+	if( mode & O_CREAT )
+		vips__set_create_time( fd ); 
+
 	g_free( path16 );
 }
 #else /*!OS_WIN32*/
@@ -610,6 +634,9 @@ vips__fopen( const char *filename, const char *mode )
 
 	g_free( path16 );
 	g_free( mode16 );
+
+	if( mode[0] == 'w' )
+		vips__set_create_time( _fileno( fp ) ); 
 #else /*!OS_WIN32*/
 	fp = fopen( filename, mode );
 #endif
@@ -1806,7 +1833,6 @@ vips_flags_from_nick( const char *domain, GType type, const char *nick )
 	return( -1 );
 }
 
-
 /* Scan @buf for the first "%ns" (eg. "%12s") and substitute the 
  * lowest-numbered one for @sub. @buf is @len bytes in size.
  *
@@ -1929,3 +1955,80 @@ vips__random_add( guint32 seed, int value )
 
 	return( vips__random( seed ) ); 
 }
+
+static void *
+vips_icc_dir_once( void *null )
+{
+#ifdef OS_WIN32
+	/* From glib get_windows_directory_root()
+	 */
+	wchar_t wwindowsdir[MAX_PATH];
+
+	if( GetWindowsDirectoryW( wwindowsdir, G_N_ELEMENTS( wwindowsdir ) ) ) {
+		/* Usually X:\Windows, but in terminal server environments
+		 * might be an UNC path, AFAIK.
+		 */
+		char *windowsdir;
+
+		if( (windowsdir = g_utf16_to_utf8( wwindowsdir, 
+			-1, NULL, NULL, NULL)) ) {
+			gchar *full_path;
+
+			full_path = g_build_filename( windowsdir, 
+				"system32", "spool", "drivers", "color", NULL );
+			g_free( windowsdir );
+
+			return( (void *) full_path );
+		}
+	}
+#endif
+
+	return( (void *) VIPS_ICC_DIR );
+}
+
+const char *
+vips__icc_dir( void )
+{
+	static GOnce once = G_ONCE_INIT;
+
+	return( (const char *) g_once( &once, 
+		(GThreadFunc) vips_icc_dir_once, NULL ) );
+}
+
+#ifdef OS_WIN32
+static HMODULE vips__dll = NULL;
+#ifdef DLL_EXPORT
+BOOL WINAPI
+DllMain( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved )
+{
+	if( fdwReason == DLL_PROCESS_ATTACH )
+		vips__dll = hinstDLL;
+
+	return( TRUE );
+}
+#endif
+#endif
+
+static void *
+vips__windows_prefix_once( void *null )
+{
+	char *prefix;
+
+#ifdef OS_WIN32
+	prefix = g_win32_get_package_installation_directory_of_module( 
+		vips__dll );
+#else
+        prefix = (char *) g_getenv( "VIPSHOME" );
+#endif
+
+	return( (void *) prefix ); 
+}
+
+const char *
+vips__windows_prefix( void )
+{
+	static GOnce once = G_ONCE_INIT;
+
+	return( (const char *) g_once( &once, 
+		(GThreadFunc) vips__windows_prefix_once, NULL ) );
+}
