@@ -57,17 +57,6 @@
 #include "presample.h"
 #include "templates.h"
 
-/**
- * VipsKernel: 
- * @VIPS_KERNEL_NEAREST: nearest-neighbour
- * @VIPS_KERNEL_LINEAR: linear interpolation
- * @VIPS_KERNEL_CUBIC: cubic interpolation
- * @VIPS_KERNEL_LANCZOS2: lanczos2 interpolation
- * @VIPS_KERNEL_LANCZOS3: lanczos3 interpolation
- *
- * 1D resampling kernels. 
- */
-
 typedef struct _VipsReduceh {
 	VipsResample parent_instance;
 
@@ -112,10 +101,10 @@ vips_reduce_get_points( VipsKernel kernel, double shrink )
 		return( 1 ); 
 
 	case VIPS_KERNEL_LINEAR:
-		return( 2 ); 
+		return( rint( 2 * shrink ) + 1 ); 
 
 	case VIPS_KERNEL_CUBIC:
-		return( 4 ); 
+		return( rint( 4 * shrink ) + 1 ); 
 
 	case VIPS_KERNEL_LANCZOS2:
 		/* Needs to be in sync with calculate_coefficients_lanczos().
@@ -142,12 +131,11 @@ vips_reduce_make_mask( double *c, VipsKernel kernel, double shrink, double x )
 		break;
 
 	case VIPS_KERNEL_LINEAR:
-		c[0] = 1.0 - x;
-		c[1] = x;
+		calculate_coefficients_triangle( c, shrink, x ); 
 		break;
 
 	case VIPS_KERNEL_CUBIC:
-		calculate_coefficients_catmull( c, x ); 
+		calculate_coefficients_adaptive_catmull( c, shrink, x ); 
 		break;
 
 	case VIPS_KERNEL_LANCZOS2:
@@ -465,7 +453,7 @@ vips_reduceh_build( VipsObject *object )
 	 */
 	reduceh->n_point = 
 		vips_reduce_get_points( reduceh->kernel, reduceh->hshrink ); 
-	g_info( "%d point mask", reduceh->n_point );
+	g_info( "reduceh: %d point mask", reduceh->n_point );
 	if( reduceh->n_point > MAX_POINT ) {
 		vips_error( object_class->nickname, 
 			"%s", _( "reduce factor too large" ) );
@@ -487,6 +475,13 @@ vips_reduceh_build( VipsObject *object )
 		for( int i = 0; i < reduceh->n_point; i++ )
 			reduceh->matrixi[x][i] = reduceh->matrixf[x][i] * 
 				VIPS_INTERPOLATE_SCALE;
+
+#ifdef DEBUG
+		printf( "vips_reduceh_build: mask %d\n    ", x ); 
+		for( int i = 0; i < reduceh->n_point; i++ )
+			printf( "%d ", reduceh->matrixi[x][i] );
+		printf( "\n" ); 
+#endif /*DEBUG*/
 	}
 
 	/* Unpack for processing.
@@ -506,12 +501,12 @@ vips_reduceh_build( VipsObject *object )
 		reduceh->n_point / 2 - 1, 0, 
 		width, in->Ysize,
 		"extend", VIPS_EXTEND_COPY,
-		NULL ) )
+		(void *) NULL ) )
 		return( -1 );
 	in = t[1];
 
 	if( vips_image_pipelinev( resample->out, 
-		VIPS_DEMAND_STYLE_THINSTRIP, in, NULL ) )
+		VIPS_DEMAND_STYLE_THINSTRIP, in, (void *) NULL ) )
 		return( -1 );
 
 	/* Size output. We need to always round to nearest, so round(), not
@@ -562,7 +557,7 @@ vips_reduceh_class_init( VipsReducehClass *reduceh_class )
 	vobject_class->description = _( "shrink an image horizontally" );
 	vobject_class->build = vips_reduceh_build;
 
-	operation_class->flags = VIPS_OPERATION_SEQUENTIAL_UNBUFFERED;
+	operation_class->flags = VIPS_OPERATION_SEQUENTIAL;
 
 	VIPS_ARG_DOUBLE( reduceh_class, "hshrink", 3, 
 		_( "Hshrink" ), 
@@ -602,34 +597,9 @@ vips_reduceh_init( VipsReduceh *reduceh )
 	reduceh->kernel = VIPS_KERNEL_LANCZOS3;
 }
 
-/**
- * vips_reduceh:
- * @in: input image
- * @out: output image
- * @hshrink: horizontal reduce
- * @...: %NULL-terminated list of optional named arguments
- *
- * Optional arguments:
- *
- * * @kernel: #VipsKernel to use to interpolate (default: lanczos3)
- * * @centre: %gboolean use centre rather than corner sampling convention
- *
- * Reduce @in horizontally by a float factor. The pixels in @out are
- * interpolated with a 1D mask generated by @kernel.
- *
- * Set @centre to use centre rather than corner sampling convention. Centre
- * convention can be useful to match the behaviour of other systems. 
- *
- * This is a very low-level operation: see vips_resize() for a more
- * convenient way to resize images. 
- *
- * This operation does not change xres or yres. The image resolution needs to
- * be updated by the application. 
- *
- * See also: vips_shrink(), vips_resize(), vips_affine().
- *
- * Returns: 0 on success, -1 on error
+/* See reduce.c for the doc comment.
  */
+
 int
 vips_reduceh( VipsImage *in, VipsImage **out, double hshrink, ... )
 {
