@@ -5,8 +5,12 @@
  * 	- from vipsthumbnail.c
  * 6/1/17
  * 	- add @size parameter
+ * 4/5/17
+ * 	- add FORCE
  * 29/5/17
  * 	- don't cache (thanks tomasc)
+ * 30/8/17
+ * 	- add intent option, thanks kleisauke
  */
 
 /*
@@ -79,6 +83,7 @@ typedef struct _VipsThumbnail {
 	gboolean linear;
 	char *export_profile;
 	char *import_profile;
+	VipsIntent intent;
 
 	/* Set by subclasses to the input image.
 	 */
@@ -137,21 +142,27 @@ vips_thumbnail_finalize( GObject *gobject )
 
 /* Calculate the shrink factor, taking into account auto-rotate, the fit mode,
  * and so on.
+ *
+ * The hshrink/vshrink are the amount to shrink the input image axes by in
+ * order for the output axes (ie. after rotation) to match the required 
+ * thumbnail->width, thumbnail->height and fit mode.
  */
-static double
+static void
 vips_thumbnail_calculate_shrink( VipsThumbnail *thumbnail, 
-	int input_width, int input_height )
+	int input_width, int input_height, double *hshrink, double *vshrink )
 {
+	/* If we will be rotating, swap the target width and height.
+	 */
 	gboolean rotate = 
-		thumbnail->angle == VIPS_ANGLE_D90 || 
-		thumbnail->angle == VIPS_ANGLE_D270;
-	int width = thumbnail->auto_rotate && rotate ? 
-		input_height : input_width;
-	int height = thumbnail->auto_rotate && rotate ? 
-		input_width : input_height;
+		(thumbnail->angle == VIPS_ANGLE_D90 || 
+		 thumbnail->angle == VIPS_ANGLE_D270) &&
+		thumbnail->auto_rotate;
+	int target_width = rotate ? 
+		thumbnail->height : thumbnail->width;
+	int target_height = rotate ? 
+		thumbnail->width : thumbnail->height;
 
 	VipsDirection direction;
-	double shrink;
 
 	/* Calculate the horizontal and vertical shrink we'd need to fit the
 	 * image to the bounding box, and pick the biggest. 
@@ -159,33 +170,53 @@ vips_thumbnail_calculate_shrink( VipsThumbnail *thumbnail,
 	 * In crop mode, we aim to fill the bounding box, so we must use the
 	 * smaller axis.
 	 */
-	double horizontal = (double) width / thumbnail->width;
-	double vertical = (double) height / thumbnail->height;
+	*hshrink = (double) input_width / target_width;
+	*vshrink = (double) input_height / target_height;
 
 	if( thumbnail->crop != VIPS_INTERESTING_NONE ) {
-		if( horizontal < vertical )
+		if( *hshrink < *vshrink )
 			direction = VIPS_DIRECTION_HORIZONTAL;
 		else
 			direction = VIPS_DIRECTION_VERTICAL;
 	}
 	else {
-		if( horizontal < vertical )
+		if( *hshrink < *vshrink )
 			direction = VIPS_DIRECTION_VERTICAL;
 		else
 			direction = VIPS_DIRECTION_HORIZONTAL;
 	}
 
-	shrink = direction == VIPS_DIRECTION_HORIZONTAL ?
-		horizontal : vertical;  
+	if( thumbnail->size != VIPS_SIZE_FORCE ) {
+		if( direction == VIPS_DIRECTION_HORIZONTAL )
+			*vshrink = *hshrink;
+		else
+			*hshrink = *vshrink;
+	}
 
-	/* Restrict to only upsize, only downsize, or both.
-	 */
-	if( thumbnail->size == VIPS_SIZE_UP )
-		shrink = VIPS_MIN( 1, shrink );
-	if( thumbnail->size == VIPS_SIZE_DOWN )
-		shrink = VIPS_MAX( 1, shrink );
+	if( thumbnail->size == VIPS_SIZE_UP ) {
+		*hshrink = VIPS_MIN( 1, *hshrink );
+		*vshrink = VIPS_MIN( 1, *vshrink );
+	}
+	else if( thumbnail->size == VIPS_SIZE_DOWN ) {
+		*hshrink = VIPS_MAX( 1, *hshrink );
+		*vshrink = VIPS_MAX( 1, *vshrink );
+	}
+}
 
-	return( shrink ); 
+/* Just the common part of the shrink: the bit by which both axes must be
+ * shrunk.
+ */
+static double
+vips_thumbnail_calculate_common_shrink( VipsThumbnail *thumbnail, 
+	int width, int height )
+{
+	double hshrink;
+	double vshrink;
+
+	vips_thumbnail_calculate_shrink( thumbnail, width, height, 
+		&hshrink, &vshrink ); 
+
+	return( VIPS_MIN( hshrink, vshrink ) );
 }
 
 /* Find the best jpeg preload shrink.
@@ -193,8 +224,8 @@ vips_thumbnail_calculate_shrink( VipsThumbnail *thumbnail,
 static int
 vips_thumbnail_find_jpegshrink( VipsThumbnail *thumbnail, int width, int height )
 {
-	double shrink = 
-		vips_thumbnail_calculate_shrink( thumbnail, width, height ); 
+	double shrink = vips_thumbnail_calculate_common_shrink( thumbnail, 
+		width, height ); 
 
 	/* We can't use pre-shrunk images in linear mode. libjpeg shrinks in Y
 	 * (of YCbCR), not linear space.
@@ -230,7 +261,7 @@ vips_thumbnail_open( VipsThumbnail *thumbnail )
 	VipsThumbnailClass *class = VIPS_THUMBNAIL_GET_CLASS( thumbnail );
 
 	VipsImage *im;
-	int shrink;
+	double shrink;
 	double scale;
 
 	if( class->get_info( thumbnail ) )
@@ -239,27 +270,29 @@ vips_thumbnail_open( VipsThumbnail *thumbnail )
 	g_info( "input size is %d x %d", 
 		thumbnail->input_width, thumbnail->input_height ); 
 
-	shrink = 1;
+	shrink = 1.0;
 	scale = 1.0;
 
 	if( vips_isprefix( "VipsForeignLoadJpeg", thumbnail->loader ) ) {
 		shrink = vips_thumbnail_find_jpegshrink( thumbnail, 
 			thumbnail->input_width, thumbnail->input_height );
-		g_info( "loading jpeg with factor %d pre-shrink", shrink ); 
+
+		g_info( "loading jpeg with factor %g pre-shrink", shrink ); 
 	}
 	else if( vips_isprefix( "VipsForeignLoadPdf", thumbnail->loader ) ||
 		vips_isprefix( "VipsForeignLoadSvg", thumbnail->loader ) ) {
-		scale = 1.0 / vips_thumbnail_calculate_shrink( thumbnail, 
-			thumbnail->input_width, thumbnail->input_height ); 
+		scale = 1.0 / vips_thumbnail_calculate_common_shrink( thumbnail, 
+			thumbnail->input_width, thumbnail->input_height );
+
 		g_info( "loading PDF/SVG with factor %g pre-scale", scale ); 
 	}
 	else if( vips_isprefix( "VipsForeignLoadWebp", thumbnail->loader ) ) {
 		shrink = VIPS_MAX( 1.0, 
-			vips_thumbnail_calculate_shrink( thumbnail, 
+			vips_thumbnail_calculate_common_shrink( thumbnail, 
 				thumbnail->input_width, 
 				thumbnail->input_height ) ); 
 
-		g_info( "loading webp with factor %d pre-shrink", shrink ); 
+		g_info( "loading webp with factor %g pre-shrink", shrink ); 
 	}
 
 	if( !(im = class->open( thumbnail, shrink, scale )) )
@@ -277,7 +310,8 @@ vips_thumbnail_build( VipsObject *object )
 		VIPS_INTERPRETATION_scRGB : VIPS_INTERPRETATION_sRGB; 
 
 	VipsImage *in;
-	double shrink;
+	double hshrink;
+	double vshrink;
 
 	/* TRUE if we've done the import of an ICC transform and still need to
 	 * export.
@@ -343,6 +377,7 @@ vips_thumbnail_build( VipsObject *object )
 		if( vips_icc_import( in, &t[1], 
 			"input_profile", thumbnail->import_profile,
 			"embedded", TRUE,
+			"intent", thumbnail->intent,
 			"pcs", VIPS_PCS_XYZ,
 			NULL ) )  
 			return( -1 );
@@ -378,13 +413,11 @@ vips_thumbnail_build( VipsObject *object )
 		in = t[3];
 	}
 
-	shrink = vips_thumbnail_calculate_shrink( thumbnail, 
-		in->Xsize, in->Ysize );
+	vips_thumbnail_calculate_shrink( thumbnail, 
+		in->Xsize, in->Ysize, &hshrink, &vshrink );
 
-	/* Use centre convention to better match imagemagick.
-	 */
-	if( vips_resize( in, &t[4], 1.0 / shrink, 
-		"centre", TRUE, 
+	if( vips_resize( in, &t[4], 1.0 / hshrink, 
+		"vscale", 1.0 / vshrink, 
 		NULL ) ) 
 		return( -1 );
 	in = t[4];
@@ -409,6 +442,7 @@ vips_thumbnail_build( VipsObject *object )
 			g_info( "exporting to device space with a profile" );
 			if( vips_icc_export( in, &t[7], 
 				"output_profile", thumbnail->export_profile,
+				"intent", thumbnail->intent,
 				NULL ) )  
 				return( -1 );
 			in = t[7];
@@ -437,6 +471,7 @@ vips_thumbnail_build( VipsObject *object )
 
 			if( vips_icc_transform( in, &t[7], 
 				thumbnail->export_profile,
+				"intent", thumbnail->intent,
 				"embedded", TRUE,
 				NULL ) ) {
 				g_warning( _( "unable to import with "
@@ -456,6 +491,7 @@ vips_thumbnail_build( VipsObject *object )
 			if( vips_icc_transform( in, &t[7], 
 				thumbnail->export_profile,
 				"input_profile", thumbnail->import_profile,
+				"intent", thumbnail->intent,
 				"embedded", FALSE,
 				NULL ) )  
 				return( -1 );
@@ -597,6 +633,13 @@ vips_thumbnail_class_init( VipsThumbnailClass *class )
 		G_STRUCT_OFFSET( VipsThumbnail, export_profile ),
 		NULL ); 
 
+	VIPS_ARG_ENUM( class, "intent", 120, 
+		_( "Intent" ), 
+		_( "Rendering intent" ),
+		VIPS_ARGUMENT_OPTIONAL_INPUT,
+		G_STRUCT_OFFSET( VipsThumbnail, intent ),
+		VIPS_TYPE_INTENT, VIPS_INTENT_RELATIVE );
+
 }
 
 static void
@@ -605,6 +648,7 @@ vips_thumbnail_init( VipsThumbnail *thumbnail )
 	thumbnail->width = 1;
 	thumbnail->height = 1;
 	thumbnail->auto_rotate = TRUE;
+	thumbnail->intent = VIPS_INTENT_RELATIVE;
 }
 
 typedef struct _VipsThumbnailFile {
@@ -650,8 +694,11 @@ vips_thumbnail_file_open( VipsThumbnail *thumbnail, int shrink, double scale )
 {
 	VipsThumbnailFile *file = (VipsThumbnailFile *) thumbnail;
 
-	/* We can't use UNBUFERRED safely on very-many-core systems.
+	/* If both shrink and scale have been set, something is wrong. It
+	 * should be one or the other (or neither).
 	 */
+	g_assert( shrink == 1 || scale == 1.0 );
+
 	if( shrink != 1 ) 
 		return( vips_image_new_from_file( file->filename, 
 			"access", VIPS_ACCESS_SEQUENTIAL,
@@ -701,19 +748,20 @@ vips_thumbnail_file_init( VipsThumbnailFile *file )
 /**
  * vips_thumbnail:
  * @filename: file to read from
- * @out: output image
+ * @out: (out): output image
  * @width: target width in pixels
  * @...: %NULL-terminated list of optional named arguments
  *
  * Optional arguments:
  *
  * * @height: %gint, target height in pixels
- * * @size: #VipsSize, upsize, downsize or both
+ * * @size: #VipsSize, upsize, downsize, both or force
  * * @auto_rotate: %gboolean, rotate upright using orientation tag
  * * @crop: #VipsInteresting, shrink and crop to fill target
  * * @linear: %gboolean, perform shrink in linear light
  * * @import_profile: %gchararray, fallback import ICC profile
  * * @export_profile: %gchararray, export ICC profile
+ * * @intent: #VipsIntent, rendering intent
  *
  * Make a thumbnail from a file. Shrinking is done in three stages: using any
  * shrink-on-load features available in the file import library, using a block
@@ -730,19 +778,22 @@ vips_thumbnail_file_init( VipsThumbnailFile *file )
  * @height rectangle, with any excess cropped away. See vips_smartcrop() for
  * details on the cropping strategy.
  *
- * Normally the operation will upsize or downsize as required. If @size is set
+ * Normally the operation will upsize or downsize as required to fit the image
+ * inside or outside the target size. If @size is set
  * to #VIPS_SIZE_UP, the operation will only upsize and will just
  * copy if asked to downsize. 
  * If @size is set
  * to #VIPS_SIZE_DOWN, the operation will only downsize and will just
  * copy if asked to upsize. 
+ * If @size is #VIPS_SIZE_FORCE, the image aspect ratio will be broken and the
+ * image will be forced to fit the target. 
  *
  * Normally any orientation tags on the input image (such as EXIF tags) are
  * interpreted to rotate the image upright. If you set @auto_rotate to %FALSE,
  * these tags will not be interpreted.
  *
  * Shrinking is normally done in sRGB colourspace. Set @linear to shrink in 
- * linear light colourspace instead --- this can give better results, but can
+ * linear light colourspace instead. This can give better results, but can
  * also be far slower, since tricks like JPEG shrink-on-load cannot be used in
  * linear space.
  *
@@ -752,6 +803,9 @@ vips_thumbnail_file_init( VipsThumbnailFile *file )
  * input image has no ICC profile, or if the profile embedded in the 
  * input image is broken.
  *
+ * Use @intent to set the rendering intent for any ICC transform. The default
+ * is #VIPS_INTENT_RELATIVE.
+ *
  * See also: vips_thumbnail_buffer().
  *
  * Returns: 0 on success, -1 on error.
@@ -868,21 +922,22 @@ vips_thumbnail_buffer_init( VipsThumbnailBuffer *buffer )
 
 /**
  * vips_thumbnail_buffer:
- * @buf: memory area to load
- * @len: size of memory area
- * @out: output image
+ * @buf: (array length=len) (element-type guint8): memory area to load
+ * @len: (type gsize): size of memory area
+ * @out: (out): output image
  * @width: target width in pixels
  * @...: %NULL-terminated list of optional named arguments
  *
  * Optional arguments:
  *
  * * @height: %gint, target height in pixels
- * * @size: #VipsSize, upsize, downsize or both
+ * * @size: #VipsSize, upsize, downsize, both or force
  * * @auto_rotate: %gboolean, rotate upright using orientation tag
  * * @crop: #VipsInteresting, shrink and crop to fill target
  * * @linear: %gboolean, perform shrink in linear light
- * * @import_probuffer: %gchararray, fallback import ICC probuffer
- * * @export_probuffer: %gchararray, export ICC probuffer
+ * * @import_profile: %gchararray, fallback import ICC profile
+ * * @export_profile: %gchararray, export ICC profile
+ * * @intent: #VipsIntent, rendering intent
  *
  * Exacty as vips_thumbnail(), but read from a memory buffer. 
  *
@@ -909,3 +964,111 @@ vips_thumbnail_buffer( void *buf, size_t len, VipsImage **out, int width, ... )
 
 	return( result );
 }
+
+typedef struct _VipsThumbnailImage {
+	VipsThumbnail parent_object;
+
+	VipsImage *in;
+} VipsThumbnailImage;
+
+typedef VipsThumbnailClass VipsThumbnailImageClass;
+
+G_DEFINE_TYPE( VipsThumbnailImage, vips_thumbnail_image, 
+	vips_thumbnail_get_type() );
+
+/* Get the info from a image.
+ */
+static int
+vips_thumbnail_image_get_info( VipsThumbnail *thumbnail )
+{
+	VipsThumbnailImage *image = (VipsThumbnailImage *) thumbnail;
+
+	/* Doesn't really matter what we put here.
+	 */
+	thumbnail->loader = "image source";
+
+	thumbnail->input_width = image->in->Xsize;
+	thumbnail->input_height = image->in->Ysize;
+	thumbnail->angle = vips_autorot_get_angle( image->in );
+
+	return( 0 );
+}
+
+/* Open an image. We can't pre-shrink with an image source, sadly.
+ */
+static VipsImage *
+vips_thumbnail_image_open( VipsThumbnail *thumbnail, 
+	int shrink, double scale )
+{
+	VipsThumbnailImage *image = (VipsThumbnailImage *) thumbnail;
+
+	g_object_ref( image->in ); 
+
+	return( image->in ); 
+}
+
+static void
+vips_thumbnail_image_class_init( VipsThumbnailClass *class )
+{
+	GObjectClass *gobject_class = G_OBJECT_CLASS( class );
+	VipsObjectClass *vobject_class = VIPS_OBJECT_CLASS( class );
+	VipsThumbnailClass *thumbnail_class = VIPS_THUMBNAIL_CLASS( class );
+
+	gobject_class->set_property = vips_object_set_property;
+	gobject_class->get_property = vips_object_get_property;
+
+	vobject_class->nickname = "thumbnail_image";
+	vobject_class->description = _( "generate thumbnail from image" );
+
+	thumbnail_class->get_info = vips_thumbnail_image_get_info;
+	thumbnail_class->open = vips_thumbnail_image_open;
+
+	VIPS_ARG_IMAGE( class, "in", 1, 
+		_( "Input" ), 
+		_( "Input image argument" ),
+		VIPS_ARGUMENT_REQUIRED_INPUT,
+		G_STRUCT_OFFSET( VipsThumbnailImage, in ) );
+
+}
+
+static void
+vips_thumbnail_image_init( VipsThumbnailImage *image )
+{
+}
+
+/**
+ * vips_thumbnail_image: (method)
+ * @in: input image
+ * @out: (out): output image
+ * @width: target width in pixels
+ * @...: %NULL-terminated list of optional named arguments
+ *
+ * Optional arguments:
+ *
+ * * @height: %gint, target height in pixels
+ * * @size: #VipsSize, upsize, downsize, both or force
+ * * @auto_rotate: %gboolean, rotate upright using orientation tag
+ * * @crop: #VipsInteresting, shrink and crop to fill target
+ * * @linear: %gboolean, perform shrink in linear light
+ * * @import_profile: %gchararray, fallback import ICC profile
+ * * @export_profile: %gchararray, export ICC profile
+ * * @intent: #VipsIntent, rendering intent
+ *
+ * Exacty as vips_thumbnail(), but read from an existing image.
+ *
+ * See also: vips_thumbnail().
+ *
+ * Returns: 0 on success, -1 on error.
+ */
+int
+vips_thumbnail_image( VipsImage *in, VipsImage **out, int width, ... )
+{
+	va_list ap;
+	int result;
+
+	va_start( ap, width );
+	result = vips_call_split( "thumbnail_image", ap, in, out, width );
+	va_end( ap );
+
+	return( result );
+}
